from django.apps import AppConfig


class DjangoPymissiveConfig(AppConfig):
    """Django Pymissive app configuration."""

    default_auto_field = "django.db.models.BigAutoField"
    name = "django_pymissive"
    verbose_name = "Django Missive"

    def ready(self):
        """Imports signal handlers when app is ready."""
        from . import signals  # noqa: F401



from django.db import models
from django.db.models.expressions import Subquery, OuterRef
from django.db.models import F, Max
from django.db.models.functions import Coalesce

from ..models.choices import MissiveSupport 

class MissiveRecipientManager(models.Manager):
    """Manager for the MissiveRecipient model."""

    def last_event_subquery(self, field: str = "event"):
        from ..models.event import MissiveEvent

        return Subquery(
            MissiveEvent.objects.filter(
                missive_id=OuterRef("missive_id"),
                recipient_id=OuterRef("id"),
            )
            .order_by("-occurred_at", "-id")
            .values("event")[:1],
            output_field=models.CharField(),
        )

    def get_queryset(self):
        qs = super().get_queryset()
        qs = qs.select_related("missive")
        qs = qs.prefetch_related("to_recipientevent")
        qs = qs.annotate(
            count_event=models.Count(
                "to_recipientevent",
            ),
            last_event=self.last_event_subquery(field="event"),
            last_event_reason=self.last_event_subquery(field="reason"),
            last_event_date=Coalesce(
                Max("to_recipientevent__occurred_at"), F("created_at")
            ),
        )
        return qs


class MissiveRecipientEmailManager(models.Manager):
    """Manager for the MissiveRecipientEmail model."""

    def get_queryset(self):
        qs = super().get_queryset()
        qs = qs.filter(recipient_support=MissiveSupport.EMAIL)
        return qs

class MissiveRecipientPhoneManager(models.Manager):
    """Manager for the MissiveRecipientPhone model."""

    def get_queryset(self):
        qs = super().get_queryset()
        qs = qs.filter(recipient_support=MissiveSupport.PHONE)
        return qs

class MissiveRecipientAddressManager(models.Manager):
    """Manager for the MissiveRecipientAddress model."""

    def get_queryset(self):
        qs = super().get_queryset()
        qs = qs.filter(recipient_support=MissiveSupport.ADDRESS)
        return qs

class MissiveRecipientApplicationManager(models.Manager):
    """Manager for the MissiveRecipientApplication model."""

    def get_queryset(self):
        qs = super().get_queryset()
        qs = qs.filter(recipient_support=MissiveSupport.APPLICATION)
        return qs



"""Missive model choices."""

from typing import Optional

from django.db import models
from django.utils.translation import gettext_lazy as _
from django.conf import settings

from pymissive.config import (
    MISSIVE_ACKNOWLEDGEMENT_LEVELS,
    MISSIVE_TYPES,
    SUCCESSFUL_EVENTS,
    INFO_EVENTS,
    FAILED_EVENTS,
    GENERIC_SUPPORT,
    DELIVERY_MODES,
    PRIORITIES,
)


def is_enable_setting(setting):
    return getattr(settings, f"MISSIVE_{setting}".upper(), True)


# Python UPPERCASE, DB lowercase (config key), label capitalize
MissiveSupport = models.TextChoices(
    "MissiveSupport",
    {k.upper(): (k, _(k.capitalize())) for k in GENERIC_SUPPORT},
)


def get_missive_support_from_type(missive_type: str) -> str:
    """Get the missive support from the type. Returns DB value (lowercase)."""
    if not missive_type:
        return ""
    mt = str(missive_type).lower()
    for key, values in GENERIC_SUPPORT.items():
        if mt in [str(v).lower() for v in values]:
            return key
    return ""


_MISSIVE_EVENT_STYLE_MAP = {
    **{k: "success" for k in SUCCESSFUL_EVENTS.keys()},
    **{k: "info" for k in INFO_EVENTS.keys()},
    **{k: "danger" for k in FAILED_EVENTS.keys()},
}


class MissiveStatus(models.TextChoices):
    """High-level missive workflow status."""

    DRAFT = "draft", _("Draft")
    PROCESSING = "processing", _("Processing")
    SUCCESS = "success", _("Success")
    FAILED = "failed", _("Failed")
    PARTIALLY_SUCCESS = "partially_success", _("Partially success")
    PARTIALLY_FAILED = "partially_failed", _("Partially failed")
    ERROR = "error", _("Error")
    CANCELLED = "cancelled", _("Cancelled")


# Python UPPERCASE, DB lowercase, label from config (already localized in translation_catalog)
def _event_key(k):
    return k.upper().replace("-", "_").replace(" ", "_")

def _event_label(v):
    """Extract translatable label from config value (str or (label, description) tuple)."""
    return v[0] if isinstance(v, tuple) else v


def _event_reason(v):
    """Extract reason from config value (second element of tuple, or empty string)."""
    return v[1] if isinstance(v, tuple) and len(v) > 1 else ""


# Map event value (DB) -> translatable reason for get_description()
_EVENT_REASONS = {
    **{k: _(v[1]) if isinstance(v, tuple) and len(v) > 1 else "" for k, v in SUCCESSFUL_EVENTS.items()},
    **{k: _(v[1]) if isinstance(v, tuple) and len(v) > 1 else "" for k, v in INFO_EVENTS.items()},
    **{k: _(v[1]) if isinstance(v, tuple) and len(v) > 1 else "" for k, v in FAILED_EVENTS.items()},
}

MissiveEventType = models.TextChoices(
    "MissiveEventType",
    {
        **{_event_key(k): (k.lower(), _(_event_label(v))) for k, v in SUCCESSFUL_EVENTS.items()},
        **{_event_key(k): (k.lower(), _(_event_label(v))) for k, v in INFO_EVENTS.items()},
        **{_event_key(k): (k.lower(), _(_event_label(v))) for k, v in FAILED_EVENTS.items()},
    },
)


def _get_event_reason(event_value: Optional[str]) -> str:
    """Return config reason for event, or empty string if unknown."""
    return _EVENT_REASONS.get(event_value or "", "")


MissiveEventType.get_description = classmethod(
    lambda cls, event_value: _get_event_reason(event_value)
)

# Python UPPERCASE, DB lowercase (from config), label capitalize
MissiveDeliveryMode = models.TextChoices(
    "MissiveDeliveryMode",
    {
        **{k.upper(): (k, _(k.capitalize())) for k in DELIVERY_MODES if is_enable_setting(k)},
    },
)

MissivePriority = models.TextChoices(
    "MissivePriority",
    {
        **{k.upper(): (k, _(k.capitalize())) for k in PRIORITIES if is_enable_setting(k)},
    },
)


# Styles by DB value (lowercase) or legacy uppercase
MISSIVE_STYLE_MAP = {
    **_MISSIVE_EVENT_STYLE_MAP,
    "draft": "secondary",
    "processing": "info",
    "success": "success",
    "failed": "warning",
    "partially_success": "info",
    "partially_failed": "warning",
    "error": "danger",
    "low": "info",
    "normal": "secondary",
    "high": "warning",
    "urgent": "danger",
    "economic": "secondary",
    "premium": "info",
    "express": "warning",
    "history": "secondary",
    "message": "primary",
    "missive": "info",
}


def get_missive_style(name: str) -> str:
    """Return the style associated with a name. Accepts DB value (lowercase) or legacy uppercase."""
    return MISSIVE_STYLE_MAP.get(name) or MISSIVE_STYLE_MAP.get((name or "").lower(), "info")


def event_to_status(event: Optional[str]) -> str:
    """Map MissiveEventType to MissiveStatus."""
    if not event:
        return MissiveStatus.DRAFT
    if event in SUCCESSFUL_EVENTS:
        return MissiveStatus.SUCCESS
    if event in FAILED_EVENTS:
        return MissiveStatus.FAILED
    if event == "draft":
        return MissiveStatus.DRAFT
    return MissiveStatus.PROCESSING


def status_from_event_counts(success_count: int, processing_count: int, failed_count: int) -> str:
    """Derive MissiveStatus from counts of last events per recipient/missive."""
    total = success_count + processing_count + failed_count
    if total == 0:
        return MissiveStatus.DRAFT
    if failed_count == total:
        return MissiveStatus.FAILED
    if failed_count > 0 and success_count > 0:
        return MissiveStatus.PARTIALLY_FAILED
    if failed_count > 0:
        return MissiveStatus.FAILED
    if success_count == total:
        return MissiveStatus.SUCCESS
    if success_count > 0 and processing_count > 0:
        return MissiveStatus.PARTIALLY_SUCCESS
    return MissiveStatus.PROCESSING


# Python UPPERCASE, DB lowercase (from config key), label from config
MissiveType = models.TextChoices(
    "MissiveType",
    {
        **{k.upper().replace("-", "_"): (k, _(v)) for k, v in MISSIVE_TYPES.items()},
    },
)


class WebhookScheme(models.TextChoices):
    """HTTP scheme for webhook URL."""

    HTTPS = "https", _("HTTPS")
    HTTP = "http", _("HTTP")


AcknowledgementLevel = models.TextChoices(
    "AcknowledgementLevel", 
    {
        **{level["name"].upper(): (level["name"], _(level["display_name"])) 
        for level in MISSIVE_ACKNOWLEDGEMENT_LEVELS if is_enable_setting(level["name"])},
    }
)



class MissiveRecipientType(models.TextChoices):
    """Recipient types."""

    RECIPIENT = "recipient", _("Recipient")
    CC = "cc", _("CC")
    BCC = "bcc", _("BCC")


class MessageDirection(models.TextChoices):
    """Message direction in an exchange."""

    INBOUND = "inbound", _("Inbound")
    OUTBOUND = "outbound", _("Outbound")


class MissiveAttachmentType(models.TextChoices):
    """Attachment types."""

    VIRTUAL_ATTACHMENT = "virtual_attachment", _("Virtual Attachment")
    ATTACHMENT = "attachment", _("Attachment")
    SIGNATURE = "signature", _("Signature")
    RECEIPT = "receipt", _("Receipt")
    PROOF = "proof", _("Proof")
    OTHER = "other", _("Other")


class MissiveThreadType(models.TextChoices):
    """Thread types."""

    MISSIVE = "missive", _("Missive")
    MESSAGE = "message", _("Message")
    HISTORY = "history", _("History")


from django.db import models

from ..models.choices import MissiveAttachmentType


class MissiveBaseAttachmentManager(models.Manager):
    """Manager for the MissiveAttachment model."""

    def get_queryset(self):
        qs = super().get_queryset()
        qs = qs.select_related("missive", "campaign")
        return qs


class MissiveAttachmentManager(MissiveBaseAttachmentManager):
    """Manager for the MissiveAttachment model."""

    def get_queryset(self):
        qs = super().get_queryset()
        qs = qs.filter(
            attachment_type=MissiveAttachmentType.ATTACHMENT,
            campaign__isnull=True,
            missive__isnull=False,
        )
        return qs


class MissiveVirtualAttachmentManager(MissiveBaseAttachmentManager):
    """Manager for the MissiveVirtualAttachment model."""

    def get_queryset(self):
        qs = super().get_queryset()
        qs = qs.filter(
            attachment_type=MissiveAttachmentType.VIRTUAL_ATTACHMENT,
            campaign__isnull=True,
            missive__isnull=False,
        )
        return qs


class CampaignAttachmentManager(MissiveBaseAttachmentManager):
    """Manager for the CampaignAttachment model."""

    def get_queryset(self):
        qs = super().get_queryset()
        qs = qs.filter(
            attachment_type=MissiveAttachmentType.ATTACHMENT,
            campaign__isnull=False,
            missive__isnull=True,
        )
        return qs


class CampaignVirtualAttachmentManager(MissiveBaseAttachmentManager):
    """Manager for the CampaignVirtualAttachment model."""

    def get_queryset(self):
        qs = super().get_queryset()
        qs = qs.filter(
            attachment_type=MissiveAttachmentType.VIRTUAL_ATTACHMENT,
            campaign__isnull=False,
            missive__isnull=True,
        )
        return qs

class MissiveProofManager(MissiveBaseAttachmentManager):
    """Manager for the MissiveProof model."""

    def get_queryset(self):
        qs = super().get_queryset()
        qs = qs.filter(
            attachment_type=MissiveAttachmentType.PROOF,
        )
        return qs


"""Admin for Missive model."""

import mimetypes

from django.contrib import admin
from django.http import HttpResponse
from django.urls import reverse
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _
from django.utils.text import format_lazy
from django_boosted import AdminBoostModel
from urllib.parse import unquote
from django.contrib import messages
from django.shortcuts import redirect
from phonenumber_field.modelfields import PhoneNumberField
from phonenumber_field.formfields import PhoneNumberField as PhoneNumberFormField
from phonenumber_field.formfields import SplitPhoneNumberField
from urllib.parse import urlencode
from ..models.missive import Missive
from ..models.recipient import MissiveRecipient
from .recipient import (
    MissiveRecipientEmailInline,
    MissiveRecipientPhoneInline,
    MissiveRecipientAddressInline,
    MissiveRecipientApplicationInline,
)
from .attachment import (
    MissiveAttachmentInline,
    MissiveVirtualAttachmentInline,
    MissiveProofInline,
)
from ..models.attachment import MissiveBaseAttachment
from ..utils import recalculate_attachment_priorities
from .event import MissiveEventInline
from .billing import MissiveBillingInline
from .related_object import MissiveRelatedObjectInline
from ..models.choices import get_missive_style, MissiveStatus, MissiveThreadType, MissiveSupport
from django_boosted import admin_boost_view, admin_boost_action

class IsBillableListFilter(admin.SimpleListFilter):
    """Custom filter for is_billable annotation (not a model field)."""

    title = _("Is billable")
    parameter_name = "is_billable"

    def lookups(self, request, model_admin):
        return [
            ("1", _("Yes")),
            ("0", _("No")),
        ]

    def queryset(self, request, queryset):
        if self.value() == "1":
            return queryset.filter(is_billable=True)
        if self.value() == "0":
            return queryset.filter(is_billable=False)
        return queryset


class IsBilledListFilter(admin.SimpleListFilter):
    """Custom filter for is_billed annotation (not a model field)."""

    title = _("Is billed")
    parameter_name = "is_billed"

    def lookups(self, request, model_admin):
        return [
            ("1", _("Yes")),
            ("0", _("No")),
        ]

    def queryset(self, request, queryset):
        if self.value() == "1":
            return queryset.filter(is_billed=True)
        if self.value() == "0":
            return queryset.filter(is_billed=False)
        return queryset


class HistoryOrMessageListFilter(admin.SimpleListFilter):
    """Custom filter for history_or_message annotation (not a model field)."""

    title = _("Thread Type")
    parameter_name = "thread_type"

    def lookups(self, request, model_admin):
        return [
            ("history", _("History")),
            ("message", _("Message")),
            ("all", _("All")),
        ]

    def choices(self, changelist):
        yield {
            "selected": self.value() is None,
            "query_string": changelist.get_query_string(remove=[self.parameter_name]),
            "display": _("Missives"),
        }

        for lookup, title in self.lookup_choices:
            yield {
                "selected": self.value() == str(lookup),
                "query_string": changelist.get_query_string({self.parameter_name: lookup}),
                "display": title,
            }

    def queryset(self, request, queryset):
        if self.value() == "history":
            return queryset.filter(thread_type=MissiveThreadType.HISTORY)
        if self.value() == "message":
            return queryset.filter(thread_type=MissiveThreadType.MESSAGE)
        if self.value() == "all":
            return queryset
        return queryset.filter(thread_type=MissiveThreadType.MISSIVE)

@admin.register(Missive)
class MissiveAdmin(AdminBoostModel):
    """Admin for missive model."""

    list_display = [
        "recipient_display",
        "sender_display",
        "provider_display",
        "campaign_display",
        "status_display",
        "event_display",
        "thread_display",
        "billing_display",
    ]
    list_filter = [
        "missive_type",
        "status",
        "priority",
        IsBillableListFilter,
        IsBilledListFilter,
        HistoryOrMessageListFilter,
        "provider",
        "created_at",
    ]
    search_fields = [
        "subject",
        "to_missiverecipient__name",
        "to_missiverecipient__email",
        "to_missiverecipient__phone",
        "to_missiverecipient__address",
        "external_id",
    ]
    readonly_fields = [
        "missive_support",
        "created_at",
        "updated_at",
        "external_id",
        "buttons_show_and_preview",
        "external_id_display",
        "total_billed_amount_display",
        "total_billing_amount_display",
        "total_estimate_amount_display",
        "is_billable_display",
        "is_billed_display",
        "billing_display",
        "thread_display",
        "thread_id",
        "thread_type",
    ]
    raw_id_fields = [
        "campaign",
    ]
    inlines = [
        MissiveRecipientEmailInline,
        MissiveRecipientPhoneInline,
        MissiveRecipientAddressInline,
        MissiveRecipientApplicationInline,
        MissiveAttachmentInline,
        MissiveVirtualAttachmentInline,
        MissiveBillingInline,
        MissiveEventInline,
        MissiveRelatedObjectInline,
        MissiveProofInline,
    ]

    def save_formset(self, request, form, formset, change):
        super().save_formset(request, form, formset, change)
        if formset.model and issubclass(formset.model, MissiveBaseAttachment):
            self._recalculate_attachment_priorities(formset, form.instance)

    def _recalculate_attachment_priorities(self, formset, parent):
        """Recalculate attachment priorities after inline save (admin bypasses model save logic)."""
        recalculate_attachment_priorities(missive_id=parent.pk if parent else None)

    def get_readonly_fields(self, request, obj=None):
        """Make all fields readonly if missive has events."""
        readonly = list(super().get_readonly_fields(request, obj))

        if obj and obj.pk and obj.external_id:
            has_events = obj.to_missiveevent.exists()
            if has_events:
                all_fields = [
                    f.name
                    for f in self.model._meta.get_fields()
                    if (not f.is_relation or f.one_to_one) and f.name not in ["id"]
                ]
                readonly = list(set(readonly + all_fields))

        return readonly

    def formfield_for_dbfield(self, db_field, request, **kwargs):
        if isinstance(db_field, PhoneNumberField):
            kwargs.setdefault("required", False)
            # Use standard PhoneNumberField for nullable fields: SplitPhoneNumberField
            # displays "None" when initial is None (django-phonenumber-field quirk)
            if db_field.null:
                return PhoneNumberFormField(**kwargs)
            return SplitPhoneNumberField(**kwargs)
        return super().formfield_for_dbfield(db_field, request, **kwargs)

    def recipient_display(self, obj):
        recipient = obj.first_recipient
        if isinstance(recipient, MissiveRecipient):
            text = recipient.name
            if obj.count_recipient > 1:
                text += f" (+{obj.count_recipient - 1})"
            return self.format_with_help_text(text, recipient.target)
        return self.format_label(_("No recipient"), label_type="warning")

    recipient_display.short_description = _("Recipient")

    def billing_display(self, obj):
        if obj.total_billed_amount is not None:
            label_type = "success" if obj.is_billed else "warning"
            return self.format_label(f"{obj.total_billed_amount:.3f}", size="small", label_type=label_type)
        if obj.total_billing_amount is not None:
            label_type = "info" if obj.is_billed else "danger"
            return self.format_label(f"{obj.total_billing_amount:.3f}", size="small", label_type=label_type)
        return "-"

    billing_display.short_description = _("Billing")

    def sender_display(self, obj):
        sender = obj.get_sender()
        name = sender["name"] or ""
        if obj.missive_support == MissiveSupport.ADDRESS:
            target = obj.sender_address or ""
        else:
            target = sender[obj.missive_support.lower()] or ""
        text = name or target or _("No sender")
        return self.format_with_help_text(text, target) if target else text

    sender_display.short_description = _("Sender")

    def external_id_display(self, obj):
        if not obj.external_id:
            return "-"
        return self.format_label(obj.external_id, size="large", label_type="success")
    
    external_id_display.short_description = _("External ID")

    def provider_display(self, obj):
        if not obj.provider:
            return "-"
        return self.format_with_help_text(
            f"{obj.provider} ({obj.get_missive_type_display()})",
            obj.provider._provider.display_name,
        )

    provider_display.short_description = _("Provider")

    def status_display(self, obj):
        priority_style = get_missive_style(obj.priority)
        priority_html = self.format_label(
            obj.get_priority_display(), size="small", label_type=priority_style
        )
        status_style = get_missive_style(obj.status)
        status_html = self.format_label(
            obj.get_status_display(), size="small", label_type=status_style
        )
        html = format_html("{} {}", priority_html, status_html)
        return self.format_with_help_text(html, obj.last_event_date)

    status_display.short_description = _("Status / Last Event Date")

    def thread_display(self, obj):
        message = self.format_label(
            format_lazy(_("{} message(s)"), obj.count_message or 0),
            size="small",
            label_type="primary",
        )
        history = self.format_label(
            format_lazy(_("{} history(s)"), obj.count_history or 0),
            size="small",
            label_type="secondary",
        )
        html = format_html("{} {}", message, history)
        return self.format_with_help_text(html, obj.get_thread_type_display())

    thread_display.short_description = _("Message(s)/History(s)/Thread")

    def button_show(self, obj):
        return format_html(
            '<a class="button" href="{}" target="_blank">{}</a>',
            reverse("django_pymissive:preview", args=["missive", obj.pk]),
            _("Show"),
        )

    def button_preview(self, obj):
        preview_url = reverse("django_pymissive:preview_form", args=["missive"])
        if obj.pk:
            preview_url = f"{preview_url}?pk={obj.pk}"
        return format_html(
            '<button type="submit" form="missive_form" formaction="{}" formmethod="post" formtarget="_blank" class="button" name="_preview" style="margin-left: 10px;">{}</button>',
            preview_url,
            _("Preview"),
        )

    def buttons_show_and_preview(self, obj):
        buttons_html = []
        if obj.pk:
            buttons_html.append(self.button_show(obj))
        buttons_html.append(self.button_preview(obj))
        return mark_safe(" ".join(str(btn) for btn in buttons_html))  # nosec B703 B308

    buttons_show_and_preview.short_description = _("Show and Preview")

    def event_display(self, obj):
        event_related_html = format_html(
            "{} {}",
            self.format_label(
                format_lazy(_("{} event(s)"), obj.count_event),
                size="small",
            ),
            self.format_label(
                format_lazy(_("{} related(s)"), obj.count_related_object),
                size="small",
                label_type="secondary",
            ),
        )
        return self.format_with_help_text(event_related_html, obj.subject)
    event_display.short_description = _("Event(s)/Related(s)/Subject")

    def campaign_display(self, obj):
        if obj.campaign is None:
            return "-"
        return self.format_with_help_text(
            self.format_label(obj.campaign.subject, size="small"),
            obj.last_campaign_send_date)
    campaign_display.short_description = _("Campaign / Last Send Date")

    def change_fieldsets(self):
        """Configure fieldsets for change view."""
        self.add_to_fieldset(
            None,
            [

                "provider",
                "missive_type",
                "acknowledgement",
                "delivery_mode",
                "priority",
            ],
        )
        self.add_to_fieldset(
            _("Sender"),
            ["brand_name", "sender_name", "sender_email", "sender_phone", "sender_address"],
        )
        self.add_to_fieldset(
            _("Reply-To"),
            ["reply_to_name", "reply_to_email", "reply_to_address"],
        )
        self.add_to_fieldset(
            _("Content"),
            ["subject", "body_html", "body_text", "buttons_show_and_preview"],
        )
        self.add_to_fieldset(
            _("Tracking"),
            [
                "campaign", 
                "status",
                "webhook_url",
                "external_id_display",
                "missive_support",
                "thread_id",
                "thread_type",
            ],
        )
        self.add_to_fieldset(_("Comment/Timestamps"), ["comment", "created_at", "updated_at"], classes=("wide", "collapse"))
        self.add_to_fieldset(
            _("Configs"),
            ["additional_context", "metadata", "additional_config"],
            classes=("wide", "collapse"),
        )
        self.add_to_fieldset(
            _("Billing"),
            [
                "billing_display",
                "total_billed_amount_display",
                "total_billing_amount_display",
                "total_estimate_amount_display",
                "is_billable_display",
                "is_billed_display",
            ],
        )

    def total_billed_amount_display(self, obj):
        if obj.total_billed_amount is None:
            return "-"
        label_type = "success" if obj.is_billed else "warning"
        return self.format_label(f"{obj.total_billed_amount:.3f}", size="small", label_type=label_type)
    total_billed_amount_display.short_description = _("Total Billed Amount")

    def total_billing_amount_display(self, obj):
        if obj.total_billing_amount is None:
            return "-"
        label_type = "info" if obj.is_billed else "danger"
        return self.format_label(f"{obj.total_billing_amount:.3f}", size="small", label_type=label_type)
    total_billing_amount_display.short_description = _("Total Billing Amount")

    def total_estimate_amount_display(self, obj):
        if obj.total_estimate_amount is None:
            return "-"
        return self.format_label(f"{obj.total_estimate_amount:.3f}", size="small", label_type="info")
    total_estimate_amount_display.short_description = _("Total Estimate Amount")

    def is_billable_display(self, obj):
        return obj.is_billable
    is_billable_display.short_description = _("Is billable")
    is_billable_display.boolean = True

    def is_billed_display(self, obj):
        return obj.is_billed
    is_billed_display.short_description = _("Is Billed")
    is_billed_display.boolean = True

    def provider_has_service(self, obj, service):
        service_name = f"{service}_{obj.missive_type}".lower()
        if obj.provider:
            return hasattr(obj.provider._provider, service_name)

    def is_draft(self, obj):
        return (obj and obj.pk and obj.status == MissiveStatus.DRAFT)

    def is_not_cancelled(self, obj):
        return (obj and obj.pk and obj.status != MissiveStatus.CANCELLED)

    def has_change_permission(self, request, obj=None):
        return self.is_not_cancelled(obj) and not obj.external_id

    def has_prepare_missive_permission(self, request, obj=None):
        return self.is_draft(obj) and self.provider_has_service(obj, "create") and not obj.external_id

    @admin_boost_action("prepare_missive", _("Prepare"))
    def handle_prepare_missive(self, request, object_id):
        object_id = unquote(object_id)
        obj = self.get_object(request, object_id)
        obj.prepare_missive()
        messages.success(request, _("Missive prepared successfully."))

    def has_resend_missive_permission(self, request, obj=None):
        return self.is_not_cancelled(obj) and obj.can_resend() and not self.is_draft(obj)

    @admin_boost_action("resend_missive", _("Resend"))
    def handle_resend_missive(self, request, object_id):
        object_id = unquote(object_id)
        obj = self.get_object(request, object_id)
        return redirect(reverse("admin:django_pymissive_missive_resend_missive", args=[obj.pk]))

    @admin_boost_view("confirm", _("Resend"), hidden=True)
    def resend_missive(self, request, obj, confirmed=False):
        if not confirmed:
            return {"confirm": _("Are you sure you want to resend this missive?")}
        new_missive = obj.resend_missive()
        messages.success(request, _("Missive resent successfully."))
        return redirect(reverse("admin:django_pymissive_missive_change", args=[new_missive.pk]))

    def has_send_missive_permission(self, request, obj=None):
        return self.is_draft(obj) and obj.can_send()

    @admin_boost_action("send_missive", _("Send"))
    def handle_send_missive(self, request, object_id):
        object_id = unquote(object_id)
        obj = self.get_object(request, object_id)
        return redirect(reverse("admin:django_pymissive_missive_send_missive", args=[obj.pk]))

    @admin_boost_view("confirm", _("Send"), hidden=True)
    def send_missive(self, request, obj, confirmed=False):
        if not confirmed:
            return {"confirm": _("Are you sure you want to send this missive?")}
        obj.send_missive()
        messages.success(request, _("Missive sent successfully."))
        return redirect(reverse("admin:django_pymissive_missive_change", args=[obj.pk]))

    def has_cancel_missive_permission(self, request, obj=None):
        return self.is_not_cancelled(obj) and self.provider_has_service(obj, "cancel") and obj.external_id

    @admin_boost_action("cancel_missive", _("Cancel"))
    def handle_cancel_missive(self, request, object_id):
        object_id = unquote(object_id)
        obj = self.get_object(request, object_id)
        obj.cancel_missive()
        messages.success(request, _("Missive cancelled successfully."))

    def has_retrieve_missive_permission(self, request, obj=None):
        return self.is_not_cancelled(obj) and self.provider_has_service(obj, "retrieve") and obj.external_id

    @admin_boost_action("retrieve_missive", _("Status"))
    def handle_retrieve_missive(self, request, object_id):
        object_id = unquote(object_id)
        obj = self.get_object(request, object_id)
        obj.retrieve_missive()
        messages.success(request, _("Missive status updated successfully."))

    def has_duplicate_missive_permission(self, request, obj=None):
        return obj and obj.pk

    @admin_boost_action("duplicate_missive", _("Duplicate"))
    def handle_duplicate_missive(self, request, object_id):
        """Duplicate a missive by creating a copy."""
        object_id = unquote(object_id)
        missive = self.get_object(request, object_id)
        new_missive = missive.duplicate_missive()
        messages.success(request, _("Missive duplicated successfully."))
        return redirect(reverse("admin:django_pymissive_missive_change", args=[new_missive.pk]))

    def has_set_billed_permission(self, request, obj=None):
        return obj and obj.is_billable

    @admin_boost_action("set_billed", _("Mark as paid"))
    def handle_set_billed(self, request, object_id):
        object_id = unquote(object_id)
        obj = self.get_object(request, object_id)
        obj.set_billed()
        messages.success(request, _("Missive marked as paid successfully."))

    @admin_boost_view("redirect", _("Show history"))
    def handle_history(self, request, obj):
        url = reverse("admin:django_pymissive_missive_changelist")
        data = {
            "thread_type": MissiveThreadType.HISTORY,
            "thread_id": obj.thread_id,
        }
        return url + "?" + urlencode(data)

    @admin_boost_view("redirect", _("Show conversation"))
    def handle_message(self, request, obj):
        url = reverse("admin:django_pymissive_missive_changelist")
        data = {
            "thread_type": MissiveThreadType.MESSAGE,
            "thread_id": obj.thread_id,
        }
        return url + "?" + urlencode(data)

    @admin_boost_view("message", _("Show proofs"))
    def handle_proofs(self, request, obj):
        """Display proofs as admin list (items: filename, url)."""
        proofs = obj.get_proofs()
        url_download = reverse("admin:django_pymissive_missive_download_proof", args=[obj.pk])
        html_links = [
            format_html(
                '<div><a href="{}" target="_blank">{}</a></div>',
                f"{url_download}?filename={proof['filename']}&url={proof['url']}",
                proof["filename"],
            )
            for proof in proofs
        ]
        return {"message": mark_safe(" ".join(str(link) for link in html_links))}

    @admin_boost_view("message", _("Download proofs"), hidden=True)
    def download_proof(self, request, obj):
        filename = request.GET.get("filename")
        url = request.GET.get("url")
        if not filename or not url:
            return HttpResponse(_("Missing filename or url"), status=400)
        content = obj.download_proof(**{
            "filename": filename,
            "url": url,
            "data": obj.get_serialized_data(attachments=False),
        })
        if content is None:
            messages.warning(request, _("Proof not available"))
            return redirect(reverse("admin:django_pymissive_missive_change", args=[obj.pk]))
        content_type, _ = mimetypes.guess_type(filename)
        response = HttpResponse(content, content_type=content_type or "application/octet-stream")
        response["Content-Disposition"] = 'attachment; filename="%s"' % filename.replace('"', '\\"')
        return response

    @admin_boost_view("redirect", _("Save proofs"))
    def save_proofs(self, request, obj):
        from django.core.files.base import ContentFile
        from ..models.choices import MissiveAttachmentType
        proofs = obj.get_proofs()
        for proof in proofs:
            filename = proof["filename"]
            url = proof["url"]
            content = obj.download_proof(**{
                "filename": filename,
                "url": url,
                "data": obj.get_serialized_data(attachments=False),
            })
            if content is None:
                continue
            obj.to_missiveattachment.get_or_create(
                attachment_type=MissiveAttachmentType.PROOF,
                metadata__proof_filename=filename,
                defaults={
                    "attachment_file": ContentFile(content, name=filename),
                    "metadata": {"proof_filename": filename, "proof_url": url},
                },
            )
        messages.success(request, _("Proofs saved successfully."))
        return redirect(reverse("admin:django_pymissive_missive_change", args=[obj.pk]))

    def has_get_billings_permission(self, request, obj=None):
        return obj and obj.can_billings()

    @admin_boost_action("get_billings", _("Get billings"))
    def handle_get_billings(self, request, object_id):
        object_id = unquote(object_id)
        obj = self.get_object(request, object_id)
        obj.get_billings()
        messages.success(request, _("Billings retrieved successfully."))
        return redirect(reverse("admin:django_pymissive_missive_change", args=[obj.pk]))



"""Abstract base models for django_pymissive."""

from django.db import models
from django.utils.translation import gettext_lazy as _


class CommentTimestampedModel(models.Model):
    """Abstract model providing comment, created_at and updated_at fields."""

    comment = models.TextField(
        blank=True,
        null=True,
        verbose_name=_("Comment"),
        help_text=_("Internal comment or note"),
    )
    created_at = models.DateTimeField(
        auto_now_add=True,
        editable=False,
        verbose_name=_("Created At"),
    )
    updated_at = models.DateTimeField(
        auto_now=True,
        editable=False,
        verbose_name=_("Updated At"),
    )

    class Meta:
        abstract = True



# Generated by Django 6.0.2 on 2026-03-17 09:36

import django.db.models.deletion
import django_pymissive.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("django_pymissive", "0002_alter_missivebaseattachment_attachment_file"),
    ]

    operations = [
        migrations.CreateModel(
            name="MissiveBilling",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "comment",
                    models.TextField(
                        blank=True,
                        help_text="Internal comment or note",
                        null=True,
                        verbose_name="Comment",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(auto_now_add=True, verbose_name="Created At"),
                ),
                (
                    "updated_at",
                    models.DateTimeField(auto_now=True, verbose_name="Updated At"),
                ),
                (
                    "billing_amount",
                    models.DecimalField(
                        blank=True,
                        decimal_places=4,
                        help_text="Amount billed for this missive or recipient",
                        max_digits=10,
                        null=True,
                        verbose_name="Billing Amount",
                    ),
                ),
                (
                    "estimate_amount",
                    models.DecimalField(
                        blank=True,
                        decimal_places=4,
                        help_text="Estimated amount before sending",
                        max_digits=10,
                        null=True,
                        verbose_name="Estimate Amount",
                    ),
                ),
                (
                    "is_billed",
                    models.BooleanField(
                        default=False,
                        help_text="Indicates if this record has been billed",
                        verbose_name="Billed",
                    ),
                ),
                (
                    "currency",
                    models.CharField(
                        blank=True,
                        help_text="ISO 4217 currency code (e.g. EUR, USD)",
                        max_length=3,
                        null=True,
                        verbose_name="Currency",
                    ),
                ),
                (
                    "invoice",
                    models.TextField(
                        blank=True,
                        help_text="Invoice or billing details from the provider",
                        null=True,
                        verbose_name="Invoice",
                    ),
                ),
                (
                    "trace",
                    models.JSONField(
                        blank=True,
                        help_text="Trace of the billing request",
                        null=True,
                        verbose_name="Trace",
                    ),
                ),
            ],
            options={
                "verbose_name": "Billing",
                "verbose_name_plural": "Billings",
                "ordering": ["-created_at"],
            },
        ),
        migrations.DeleteModel(
            name="MissiveRecipientNotification",
        ),
        migrations.CreateModel(
            name="MissiveRecipientApplication",
            fields=[],
            options={
                "verbose_name": "Application Recipient",
                "verbose_name_plural": "Application Recipients",
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=("django_pymissive.missiverecipient",),
        ),
        migrations.RenameField(
            model_name="missivecampaign",
            old_name="acknowledgement_postal",
            new_name="acknowledgement_lre",
        ),
        migrations.RenameField(
            model_name="missivecampaign",
            old_name="priority_postal",
            new_name="priority_lre",
        ),
        migrations.RemoveField(
            model_name="missivecampaign",
            name="body_postal",
        ),
        migrations.RemoveField(
            model_name="missivecampaign",
            name="delivery_mode_postal",
        ),
        migrations.RemoveField(
            model_name="missiveevent",
            name="billing_amount",
        ),
        migrations.RemoveField(
            model_name="missiveevent",
            name="description",
        ),
        migrations.RemoveField(
            model_name="missiveevent",
            name="estimate_amount",
        ),
        migrations.RemoveField(
            model_name="missiveevent",
            name="is_billed",
        ),
        migrations.RemoveField(
            model_name="missiveevent",
            name="provider",
        ),
        migrations.RemoveField(
            model_name="missiveevent",
            name="user_action",
        ),
        migrations.AddField(
            model_name="missivecampaign",
            name="delivery_mode_lre",
            field=models.CharField(
                choices=[
                    ("economic", "Economic"),
                    ("normal", "Normal"),
                    ("premium", "Premium"),
                    ("express", "Express"),
                ],
                default="normal",
                help_text="Delivery mode (economic, normal, premium, express)",
                max_length=50,
                verbose_name="Delivery Mode",
            ),
        ),
        migrations.AddField(
            model_name="missivecampaign",
            name="first_document",
            field=django_pymissive.fields.RichTextField(
                blank=True,
                help_text="First document content (HTML, converted to PDF for LRE)",
                verbose_name="First Document",
            ),
        ),
        migrations.AddField(
            model_name="missiveevent",
            name="client_initiated",
            field=models.BooleanField(
                default=False,
                help_text="True if the event was initiated by django_pymissive (send, resend, cancel, batch, etc.), False if received from a provider webhook.",
                verbose_name="Client Initiated",
            ),
        ),
        migrations.AddField(
            model_name="missiveevent",
            name="reason",
            field=models.TextField(
                blank=True,
                help_text="Reason or details about this event (provider-specific)",
                verbose_name="Reason",
            ),
        ),
        migrations.AlterField(
            model_name="missive",
            name="body_html",
            field=django_pymissive.fields.RichTextField(
                blank=True,
                help_text="HTML message body (email) or rich content (LRE)",
                null=True,
                verbose_name="Body HTML",
            ),
        ),
        migrations.AlterField(
            model_name="missive",
            name="delivery_mode",
            field=models.CharField(
                blank=True,
                choices=[
                    ("economic", "Economic"),
                    ("normal", "Normal"),
                    ("premium", "Premium"),
                    ("express", "Express"),
                ],
                help_text="Delivery mode (economic, normal, premium, express)",
                max_length=50,
                null=True,
                verbose_name="Delivery Mode",
            ),
        ),
        migrations.AlterField(
            model_name="missive",
            name="missive_support",
            field=models.CharField(
                choices=[
                    ("email", "Email"),
                    ("phone", "Phone"),
                    ("address", "Address"),
                    ("application", "Application"),
                ],
                editable=False,
                help_text="Support for the missive (email, phone, address, application)",
                max_length=50,
                verbose_name="Missive Support",
            ),
        ),
        migrations.AlterField(
            model_name="missive",
            name="missive_type",
            field=models.CharField(
                choices=[
                    ("lre", "Electronic registered letter (LRE)"),
                    ("ere", "Electronic registered email (ERE)"),
                    ("email", "Email"),
                    ("email_marketing", "Marketing email"),
                    ("sms", "SMS"),
                    ("rcs", "RCS (Rich SMS)"),
                    ("voice_call", "Automated voice call"),
                    ("notification", "In-app notification"),
                    ("push_notification", "Mobile push notification"),
                    ("branded", "Branded App messaging (WhatsApp, Slack, etc.)"),
                ],
                help_text="Type of missive (email, sms, lre, ere, etc.)",
                max_length=50,
                verbose_name="Missive Type",
            ),
        ),
        migrations.AlterField(
            model_name="missive",
            name="status",
            field=models.CharField(
                choices=[
                    ("draft", "Draft"),
                    ("processing", "Processing"),
                    ("success", "Success"),
                    ("failed", "Failed"),
                    ("partially_success", "Partially success"),
                    ("partially_failed", "Partially failed"),
                    ("error", "Error"),
                    ("cancelled", "Cancelled"),
                ],
                default="draft",
                help_text="Current status of the missive",
                max_length=20,
                verbose_name="Status",
            ),
        ),
        migrations.AlterField(
            model_name="missivebaseattachment",
            name="attachment_object_arguments",
            field=django_pymissive.fields.JSONField(
                blank=True,
                default={"args": [], "kwargs": {}, "method": "retrieve_attachment"},
                help_text="Arguments to pass to the file method (as dict for **kwargs)",
                verbose_name="Attachment Object Arguments",
            ),
        ),
        migrations.AlterField(
            model_name="missiveconfig",
            name="missive_type",
            field=models.CharField(
                choices=[
                    ("lre", "Electronic registered letter (LRE)"),
                    ("ere", "Electronic registered email (ERE)"),
                    ("email", "Email"),
                    ("email_marketing", "Marketing email"),
                    ("sms", "SMS"),
                    ("rcs", "RCS (Rich SMS)"),
                    ("voice_call", "Automated voice call"),
                    ("notification", "In-app notification"),
                    ("push_notification", "Mobile push notification"),
                    ("branded", "Branded App messaging (WhatsApp, Slack, etc.)"),
                ],
                help_text="Type of missive (email, sms, postal, etc.)",
                max_length=50,
                unique=True,
                verbose_name="Missive type",
            ),
        ),
        migrations.AlterField(
            model_name="missiveevent",
            name="event",
            field=models.CharField(
                blank=True,
                choices=[
                    ("delivered", "Delivered"),
                    ("opened", "Opened"),
                    ("read", "Read"),
                    ("clicked", "Clicked"),
                    ("proofs_of_delivery", "Proofs of Delivery"),
                    ("untreated", "Untreated"),
                    ("draft", "Draft"),
                    ("sent", "Sent"),
                    ("accepted", "Accepted"),
                    ("processed", "Processed"),
                    ("deposit_proof", "Deposit proof"),
                    ("proof_of_content", "Proof of content"),
                    ("archived", "Archived"),
                    ("attempted_delivery", "Attempted delivery"),
                    ("prepare", "Prepare"),
                    ("pending", "Pending"),
                    ("processing", "Processing"),
                    ("queued", "Queued"),
                    ("request", "Request"),
                    ("deferred", "Deferred"),
                    ("scheduled", "Scheduled"),
                    ("unknown", "Unknown"),
                    ("failed", "Failed"),
                    ("cancelled", "Cancelled"),
                    ("hard_bounce", "Hard bounce"),
                    ("soft_bounce", "Soft bounce"),
                    ("dropped", "Dropped"),
                    ("spam", "Spam"),
                    ("spam_report", "Spam report"),
                    ("blocked", "Blocked"),
                    ("rejected", "Rejected"),
                    ("refused", "Refused"),
                    ("invalid", "Invalid"),
                    ("carrier_rejected", "Carrier rejected"),
                    ("undelivered", "Undelivered"),
                    ("unsubscribe", "Unsubscribe"),
                    ("suppressed", "Suppressed"),
                    ("mailbox_full", "Mailbox full"),
                    ("domain_not_found", "Domain not found"),
                    ("error", "Error"),
                ],
                help_text="Event type (sent, delivered, read, failed, etc.)",
                max_length=50,
                null=True,
                verbose_name="Event",
            ),
        ),
        migrations.AlterField(
            model_name="missiverecipient",
            name="recipient_support",
            field=models.CharField(
                blank=True,
                choices=[
                    ("email", "Email"),
                    ("phone", "Phone"),
                    ("address", "Address"),
                    ("application", "Application"),
                ],
                help_text="Model of recipient",
                max_length=255,
                null=True,
                verbose_name="Recipient Model",
            ),
        ),
        migrations.AlterField(
            model_name="missiverecipient",
            name="status",
            field=models.CharField(
                choices=[
                    ("draft", "Draft"),
                    ("processing", "Processing"),
                    ("success", "Success"),
                    ("failed", "Failed"),
                    ("partially_success", "Partially success"),
                    ("partially_failed", "Partially failed"),
                    ("error", "Error"),
                    ("cancelled", "Cancelled"),
                ],
                default="draft",
                help_text="Current status of the missive",
                max_length=20,
                verbose_name="Status",
            ),
        ),
        migrations.AddField(
            model_name="missivebilling",
            name="missive",
            field=models.ForeignKey(
                help_text="Missive associated with this billing record",
                on_delete=django.db.models.deletion.CASCADE,
                related_name="to_missivebilling",
                to="django_pymissive.missive",
                verbose_name="Missive",
            ),
        ),
        migrations.AddField(
            model_name="missivebilling",
            name="recipient",
            field=models.ForeignKey(
                blank=True,
                help_text="Recipient when billing is per-recipient; null for missive-level billing",
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="to_recipientbilling",
                to="django_pymissive.missiverecipient",
                verbose_name="Recipient",
            ),
        ),
    ]



"""Preview views for Missive and MissiveCampaign models."""

from types import SimpleNamespace

from django.contrib.admin.views.decorators import staff_member_required
from django.forms import modelform_factory
from django.http import Http404, HttpResponse
from django.template.response import TemplateResponse
from django.utils.decorators import method_decorator
from django.utils.translation import gettext_lazy as _
from django.views.generic import DetailView, View

from ..models.campaign import MissiveCampaign
from ..models.choices import MissiveRecipientType
from ..models.missive import Missive


MISSIVE_TEMPLATE_MAP = {
    "email": "django_pymissive/email_preview.html",
    "email_marketing": "django_pymissive/email_preview.html",
    "ere": "django_pymissive/email_preview.html",
    "sms": "django_pymissive/sms_preview.html",
    "rcs": "django_pymissive/sms_preview.html",
    "postal": "django_pymissive/postal_preview.html",
    "postal_registered": "django_pymissive/postal_preview.html",
    "postal_signature": "django_pymissive/postal_preview.html",
    "lre": "django_pymissive/postal_preview.html",
    "lre_qualified": "django_pymissive/postal_preview.html",
}

CAMPAIGN_TEMPLATE_MAP = {
    "email": "django_pymissive/email_preview.html",
    "sms": "django_pymissive/sms_preview.html",
    "postal": "django_pymissive/postal_preview.html",
}

DEFAULT_TEMPLATE = "django_pymissive/base_preview.html"

_PREVIEW_CONFIG = {
    "missive": {"model": Missive, "template_map": MISSIVE_TEMPLATE_MAP},
    "campaign": {"model": MissiveCampaign, "template_map": CAMPAIGN_TEMPLATE_MAP},
}

def _build_form(model, post_data, pk=None):
    """Bound modelform; uses pk from args, POST id/_save, or explicit pk."""
    ModelForm = modelform_factory(model, fields="__all__")
    pk = pk or post_data.get("id") or post_data.get("_save")
    if pk:
        try:
            return ModelForm(post_data, instance=model.objects.get(pk=pk))
        except model.DoesNotExist:
            pass
    return ModelForm(post_data)


def _set_field(instance, form, field_name, value, post_data):
    """Set field via widget/clean logic or raw value."""
    field = form.fields[field_name]

    if hasattr(field, "widget") and hasattr(field.widget, "value_from_datadict"):
        widget_value = field.widget.value_from_datadict(post_data, None, field_name)
        if widget_value:
            try:
                setattr(instance, field_name, field.clean(widget_value))
                return
            except (ValueError, TypeError):
                setattr(instance, field_name, widget_value)
                return

    try:
        setattr(instance, field_name, field.clean(value))
    except (ValueError, TypeError, AttributeError):
        try:
            setattr(instance, field_name, value)
        except (ValueError, TypeError):
            pass


def _populate_from_invalid_form(model, form, post_data):
    """Instance from cleaned_data + raw POST for empty fields."""
    instance = form.instance if form.instance.pk else model()

    for field_name, value in (form.cleaned_data or {}).items():
        if value is not None:
            try:
                setattr(instance, field_name, value)
            except (ValueError, TypeError):
                pass

    for field_name in form.fields:
        value = post_data.get(field_name)
        if not value or getattr(instance, field_name, None) is not None:
            continue
        try:
            _set_field(instance, form, field_name, value, post_data)
        except (ValueError, TypeError, AttributeError):
            try:
                setattr(instance, field_name, value)
            except (ValueError, TypeError):
                pass

    return instance


def _format_recipient_email(r):
    return r.email or (str(r.phone) if r.phone else str(r.address) if r.address else "")


def _phone_from_post(post_data, prefix):
    """Phone from POST (single key or prefix_0/prefix_1)."""
    val = post_data.get(prefix, "")
    if val:
        return str(val).strip()
    region = post_data.get(f"{prefix}_0", "")
    national = post_data.get(f"{prefix}_1", "")
    if not (region and national):
        return ""
    try:
        import phonenumbers
        parsed = phonenumbers.parse(national.strip(), region.strip())
        return phonenumbers.format_number(
            parsed, phonenumbers.PhoneNumberFormat.INTERNATIONAL
        )
    except Exception:
        return f"{region} {national}".strip()


def _build_sms_context(instance, post_data=None):
    """SMS sender context; fallback to POST when instance is empty."""
    sender = getattr(instance, "sender", None) or getattr(instance, "phone_sender", {}) or {}
    ctx = {
        "sender": sender,
    }
    if post_data and (not sender.get("phone") or not sender.get("name")):
        updated = dict(sender)
        if not updated.get("phone"):
            updated["phone"] = _phone_from_post(post_data, "sender_phone")
        if not updated.get("name"):
            updated["name"] = post_data.get("sender_phone_name", "") or post_data.get("sender_name", "")
        ctx["sender"] = updated
    return ctx


def _build_context_by_type(preview_type, instance, post_data=None):
    pt = (preview_type or "").lower()
    if pt in ("email", "email_marketing", "ere"):
        return _build_email_context(instance)
    if pt in ("sms", "rcs"):
        return _build_sms_context(instance, post_data)
    if pt in ("postal", "postal_registered", "postal_signature", "lre", "lre_qualified"):
        return _build_lre_context(instance)
    return {}


def _geoaddress_lines(addr):
    """Extract displayable address lines from a GeoaddressField value."""
    if not addr:
        return []
    if isinstance(addr, str):
        return [addr] if addr.strip() else []
    if isinstance(addr, dict):
        order = [
            "address_line_1", "address_line_2", "address_line_3",
            "postal_code", "city", "state", "region", "country",
        ]
        lines = []
        for key in order:
            val = addr.get(key)
            if val and str(val).strip():
                lines.append(str(val).strip())
        if not lines:
            for val in addr.values():
                if val and str(val).strip():
                    lines.append(str(val).strip())
        return lines
    return [str(addr)]


def _build_lre_context(instance, post_data=None):
    """LRE sender/recipient/delivery context."""
    sender = getattr(instance, "sender", None) or {}
    sender_address = sender.get("address") if isinstance(sender, dict) else None
    if not sender_address:
        sender_address = getattr(instance, "sender_address", None)

    reply_to = getattr(instance, "reply_to", None)
    reply_to_address = None
    if isinstance(reply_to, dict):
        reply_to_address = reply_to.get("address")
    if not reply_to_address:
        reply_to_address = getattr(instance, "reply_to_address", None)

    recipients = []
    recipient_manager = getattr(instance, "to_missiverecipient", None)
    if recipient_manager and getattr(instance, "pk", None):
        try:
            for r in recipient_manager.filter(
                recipient_type=MissiveRecipientType.RECIPIENT
            ):
                recipients.append({
                    "name": r.name or "",
                    "address_lines": _geoaddress_lines(r.address),
                })
        except Exception:
            pass

    acknowledgement_display = ""
    delivery_mode_display = ""
    priority_display = ""
    if hasattr(instance, "get_acknowledgement_display"):
        try:
            acknowledgement_display = instance.get_acknowledgement_display() or ""
        except Exception:
            pass
    if hasattr(instance, "get_delivery_mode_display"):
        try:
            delivery_mode_display = instance.get_delivery_mode_display() or ""
        except Exception:
            pass
    if hasattr(instance, "get_priority_display"):
        try:
            priority_display = instance.get_priority_display() or ""
        except Exception:
            pass

    return {
        "sender": sender,
        "sender_address_lines": _geoaddress_lines(sender_address),
        "postal_recipients": recipients,
        "acknowledgement_display": acknowledgement_display,
        "delivery_mode_display": delivery_mode_display,
        "priority_display": priority_display,
    }


def _build_email_context(instance):
    """Email header context; recipients from to_missiverecipient when saved."""
    sender = getattr(instance, "sender", None) or getattr(instance, "email_sender", None) or {}
    reply_to = getattr(instance, "reply_to", None) or getattr(instance, "email_reply_to", None)
    context = {
        "sender": sender,
        "reply_to": reply_to,
        "to_recipients": [],
        "cc_recipients": [],
        "bcc_recipients": [],
    }

    recipient_manager = getattr(instance, "to_missiverecipient", None)
    if recipient_manager is None or not getattr(instance, "pk", None):
        return context

    try:
        for r in recipient_manager.all():
            if r.recipient_type == MissiveRecipientType.RECIPIENT:
                email = _format_recipient_email(r)
                context["to_recipients"].append({"name": r.name or "", "email": email})
            elif r.recipient_type == MissiveRecipientType.CC:
                email = _format_recipient_email(r)
                context["cc_recipients"].append({"name": r.name or "", "email": email})
            elif r.recipient_type == MissiveRecipientType.BCC:
                email = _format_recipient_email(r)
                context["bcc_recipients"].append({"name": r.name or "", "email": email})
    except Exception:
        pass

    return context


def _campaign_attachments_physical(campaign):
    if not getattr(campaign, "pk", None):
        return []
    try:
        return list(campaign.attachments_physical)
    except Exception:
        return []


def _campaign_to_missive_preview(campaign, preview_type):
    attachments_physical = _campaign_attachments_physical(campaign)
    if preview_type == "email":
        return SimpleNamespace(
            subject=campaign.subject,
            body_html=campaign.body_html,
            body_text=campaign.body_text,
            body_html_compiled=campaign.body_html_compiled,
            body_text_compiled=campaign.body_text_compiled,
            sender=campaign.email_sender or {},
            reply_to=campaign.email_reply_to,
            attachments_physical=attachments_physical,
        )
    if preview_type == "sms":
        body_sms = getattr(campaign, "body_sms", "") or ""
        return SimpleNamespace(
            body_text=body_sms,
            body_sms_compiled=body_sms,
            body_html="",
            sender=campaign.phone_sender or {},
            attachments_physical=attachments_physical,
        )
    # address / LRE
    first_document = ""
    if hasattr(campaign, "first_document_compiled"):
        first_document = campaign.first_document_compiled()
    if not first_document:
        first_document = getattr(campaign, "first_document", "") or ""
    sender = campaign.address_sender or {}
    return SimpleNamespace(
        body_html=first_document,
        body_text="",
        sender=sender,
        sender_address=getattr(campaign, "sender_address", None),
        reply_to_address=getattr(campaign, "reply_to_address", None),
        reply_to_address_name=getattr(campaign, "reply_to_address_name", "") or "",
        acknowledgement=getattr(campaign, "acknowledgement_lre", None),
        delivery_mode=getattr(campaign, "delivery_mode_lre", None),
        priority=getattr(campaign, "priority_lre", None),
        attachments_physical=attachments_physical,
    )


class PreviewView(DetailView):

    context_object_name = "missive"

    def setup(self, request, *args, **kwargs):
        super().setup(request, *args, **kwargs)
        key = kwargs.get("campaign_or_missive", "")
        config = _PREVIEW_CONFIG.get(key)
        if config is None:
            raise Http404
        self._key = key
        self.model = config["model"]
        self._template_map = config["template_map"]

    def get_preview_type(self):
        if self._key == "missive":
            return (self.object.missive_type or "").lower()
        return self.request.GET.get("type", "email").lower()

    def get_preview_object(self):
        if self._key == "campaign":
            return _campaign_to_missive_preview(self.object, self.get_preview_type())
        return self.object

    def get_template_names(self):
        return [self._template_map.get(self.get_preview_type(), DEFAULT_TEMPLATE)]

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        preview_type = self.get_preview_type()

        context["missive"] = self.get_preview_object()

        if self._key == "campaign":
            context["campaign"] = self.object
            context["title"] = _("Preview: {} ({})").format(self.object, preview_type)
        else:
            context["title"] = _("Preview: {}").format(self.object)

        type_ctx = _build_context_by_type(preview_type, self.object)
        if type_ctx:
            context.update(type_ctx)
        return context


@method_decorator(staff_member_required, name="dispatch")
class PreviewFormView(View):

    http_method_names = ["post"]

    def _get_config(self):
        key = self.kwargs.get("campaign_or_missive", "")
        config = _PREVIEW_CONFIG.get(key)
        if config is None:
            raise Http404
        return key, config

    def _get_preview_type(self, key, instance):
        if key == "missive":
            missive_type = getattr(instance, "missive_type", None) or self.request.POST.get("missive_type")
            if missive_type:
                instance.missive_type = missive_type
            return (missive_type or "").lower()
        return (
            self.request.GET.get("type")
            or self.request.POST.get("_preview_type")
            or self.request.POST.get("_preview")
            or "email"
        ).lower()

    def _get_context(self, key, instance, preview_type):
        if key == "campaign":
            return {
                "campaign": instance,
                "missive": _campaign_to_missive_preview(instance, preview_type),
                "title": _("Preview: {} ({})").format(getattr(instance, "name", None) or getattr(instance, "subject", None) or "Campaign", preview_type),
            }
        return {
            "missive": instance,
            "title": _("Preview: {}").format(preview_type or "Missive"),
        }

    def post(self, request, *args, **kwargs):
        key, config = self._get_config()
        model = config["model"]
        template_map = config["template_map"]

        form = _build_form(model, request.POST, pk=request.GET.get("pk"))
        instance = (
            form.save(commit=False)
            if form.is_valid()
            else _populate_from_invalid_form(model, form, request.POST)
        )

        preview_type = self._get_preview_type(key, instance)
        template_name = template_map.get(preview_type, DEFAULT_TEMPLATE)
        context = self._get_context(key, instance, preview_type)
        type_ctx = _build_context_by_type(preview_type, instance, post_data=request.POST)
        if type_ctx:
            context.update(type_ctx)
        return TemplateResponse(request, template_name, context)


@method_decorator(staff_member_required, name="dispatch")
class DownloadPDFView(DetailView):
    """Download the compiled body as a PDF file."""

    def setup(self, request, *args, **kwargs):
        super().setup(request, *args, **kwargs)
        key = kwargs.get("campaign_or_missive", "")
        config = _PREVIEW_CONFIG.get(key)
        if config is None:
            raise Http404
        self._key = key
        self.model = config["model"]

    def get(self, request, *args, **kwargs):
        self.object = self.get_object()

        if self._key == "missive":
            pdf_bytes = self.object.body_to_pdf()
            filename = f"missive-{self.object.pk}.pdf"
        else:
            from django.conf import settings
            from django.utils.module_loading import import_string

            pdf_generator = getattr(
                settings,
                "PYMISSIVE_PDF_GENERATOR",
                "django_pymissive.pdf.body_to_pdf",
            )
            fake_missive = _campaign_to_missive_preview(
                self.object, "postal"
            )
            fake_missive.get_locally_or_campaign_value = (
                lambda field, fallback="": getattr(fake_missive, field, fallback) or fallback
            )
            fake_missive.missive_context = lambda: getattr(
                self.object, "campaign_context", lambda: {}
            )()
            pdf_bytes = import_string(pdf_generator)(fake_missive)
            filename = f"campaign-{self.object.pk}.pdf"

        response = HttpResponse(pdf_bytes, content_type="application/pdf")
        response["Content-Disposition"] = f'attachment; filename="{filename}"'
        return response



"""Managers for django_pymissive."""

from .provider import ProviderManager
from .missive import (
    MissiveManager,
    MissiveMessageManager,
    MissiveHistoryManager,
)
from .campaign import MissiveCampaignManager
from .event import MissiveEventManager
from .attachment import (
    MissiveBaseAttachmentManager,
    MissiveAttachmentManager,
    MissiveVirtualAttachmentManager,
    CampaignAttachmentManager,
    CampaignVirtualAttachmentManager,
    MissiveProofManager,
)
from .related_object import (
    MissiveRelatedObjectManager,
    CampaignRelatedObjectManager,
)
from .recipient import (
    MissiveRecipientManager,
    MissiveRecipientEmailManager,
    MissiveRecipientPhoneManager,
    MissiveRecipientAddressManager,
    MissiveRecipientApplicationManager,
)

__all__ = [
    "ProviderManager",
    "MissiveManager",
    "MissiveMessageManager",
    "MissiveHistoryManager",
    "MissiveCampaignManager",
    "MissiveEventManager",
    "MissiveBaseAttachmentManager",
    "MissiveAttachmentManager",
    "MissiveVirtualAttachmentManager",
    "CampaignAttachmentManager",
    "CampaignVirtualAttachmentManager",
    "MissiveRelatedObjectManager",
    "CampaignRelatedObjectManager",
    "MissiveRecipientManager",
    "MissiveRecipientEmailManager",
    "MissiveRecipientPhoneManager",
    "MissiveRecipientAddressManager",
    "MissiveRecipientApplicationManager",

]



# Generated by Django 6.0.3 on 2026-03-11 17:11

import django.db.models.deletion
import django_geoaddress.fields
import django_providerkit.fields
import django_pymissive.fields
import django_pymissive.models.attachment
import django_pymissive.models.webhook
import django_pymissive.utils
import phonenumber_field.modelfields
import uuid
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='MissiveProviderModel',
            fields=[
                ('name', models.CharField(help_text='Provider name (e.g., sendgrid)', max_length=255, primary_key=True, serialize=False, verbose_name='Name')),
                ('display_name', models.CharField(help_text='Provider display name', verbose_name='Display Name')),
                ('description', models.CharField(help_text='Provider description', verbose_name='Description')),
                ('priority', models.IntegerField(help_text='Provider priority', verbose_name='Priority')),
                ('config_status_str', models.CharField(help_text='Config status', verbose_name='Config status')),
                ('config_prefix', models.CharField(help_text='Config prefix', verbose_name='Config prefix')),
                ('config_keys', models.JSONField(help_text='Config keys', verbose_name='Config keys')),
                ('missing_config_keys', models.JSONField(help_text='Missing config keys', verbose_name='Missing config keys')),
                ('is_config_ready', models.BooleanField(help_text='Is config ready', verbose_name='Is config ready')),
                ('package_status_str', models.CharField(help_text='Package status', verbose_name='Package status')),
                ('required_packages', models.JSONField(help_text='Required packages', verbose_name='Required packages')),
                ('missing_packages', models.JSONField(help_text='Missing packages', verbose_name='Missing packages')),
                ('are_packages_installed', models.BooleanField(help_text='Are packages installed', verbose_name='Are packages installed')),
                ('service_status_str', models.CharField(help_text='Service status', verbose_name='Service status')),
                ('missing_services', models.JSONField(help_text='Missing services', verbose_name='Missing services')),
                ('services', models.JSONField(help_text='Services', verbose_name='Services')),
                ('are_services_implemented', models.BooleanField(help_text='Are services implemented', verbose_name='Are services implemented')),
            ],
            options={
                'verbose_name': 'Provider',
                'verbose_name_plural': 'Providers',
                'ordering': ['-priority', 'name'],
                'managed': False,
            },
        ),
        migrations.CreateModel(
            name='MissiveServiceModel',
            fields=[
                ('name', models.CharField(help_text='Service name', max_length=255, primary_key=True, serialize=False, verbose_name='Name')),
                ('fields', models.JSONField(help_text='Service fields', verbose_name='Service fields')),
                ('description', models.CharField(help_text='Service description', verbose_name='Service description')),
            ],
            options={
                'verbose_name': 'Provider Service',
                'verbose_name_plural': 'Provider Services',
                'managed': False,
            },
        ),
        migrations.CreateModel(
            name='MissiveWebhook',
            fields=[
                ('provider', django_providerkit.fields.ProviderField(blank=True, help_text='Provider used to send this missive', max_length=100, null=True, verbose_name='Provider')),
                ('webhook_id', models.CharField(help_text='Webhook ID', max_length=255, primary_key=True, serialize=False, verbose_name='Webhook ID')),
                ('scheme', models.CharField(blank=True, choices=[('https', 'HTTPS'), ('http', 'HTTP')], default=django_pymissive.models.webhook.get_default_webhook_scheme, help_text='HTTP or HTTPS', max_length=5, verbose_name='Scheme')),
                ('domain', models.CharField(blank=True, default=django_pymissive.utils.get_default_domain, help_text='Base domain (e.g. example.com). Webhook path is built from provider and type.', max_length=255, verbose_name='Domain')),
                ('id', models.CharField(help_text='ID', verbose_name='ID')),
                ('type', models.CharField(choices=[('postal', 'Postal mail'), ('lre', 'Electronic registered letter'), ('ere', 'Electronic registered email (ERE)'), ('email', 'Email'), ('email_marketing', 'Marketing email'), ('sms', 'SMS'), ('rcs', 'RCS (Rich SMS)'), ('voice_call', 'Automated voice call'), ('notification', 'In-app notification'), ('push_notification', 'Mobile push notification'), ('branded', 'Branded App messaging (WhatsApp, Slack, etc.)')], help_text='Type', verbose_name='Type')),
                ('url', models.CharField(help_text='URL', verbose_name='URL')),
                ('description', models.CharField(help_text='Description', verbose_name='Description')),
                ('created_at', models.DateTimeField(help_text='Created At', verbose_name='Created At')),
                ('updated_at', models.DateTimeField(help_text='Updated At', verbose_name='Updated At')),
            ],
            options={
                'verbose_name': 'Webhook',
                'verbose_name_plural': 'Webhooks',
                'ordering': ['-created_at'],
                'managed': False,
            },
        ),
        migrations.CreateModel(
            name='Missive',
            fields=[
                ('comment', models.TextField(blank=True, help_text='Internal comment or note', null=True, verbose_name='Comment')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created At')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated At')),
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False, verbose_name='ID')),
                ('thread_id', models.UUIDField(db_index=True, default=uuid.uuid4, editable=False, help_text='Thread ID for the missive', verbose_name='Thread')),
                ('thread_type', models.CharField(choices=[('missive', 'Missive'), ('message', 'Message'), ('history', 'History')], default='missive', help_text='Type of thread (missive, message, history)', max_length=50, verbose_name='Thread Type')),
                ('provider', django_providerkit.fields.ProviderField(blank=True, help_text='Provider used to send this missive', max_length=100, verbose_name='Provider')),
                ('status', models.CharField(choices=[('draft', 'Draft'), ('processing', 'Processing'), ('success', 'Success'), ('failed', 'Failed'), ('error', 'Error'), ('cancelled', 'Cancelled')], default='draft', help_text='Current status of the missive', max_length=20, verbose_name='Status')),
                ('missive_support', models.CharField(choices=[('email', 'Email'), ('phone', 'Phone'), ('address', 'Address'), ('notification', 'Notification'), ('branded', 'Branded')], editable=False, help_text='Support for the missive (email, SMS, postal, etc.)', max_length=50, verbose_name='Missive Support')),
                ('brand_name', models.CharField(blank=True, help_text='Brand name used to send this missive', max_length=50, null=True, verbose_name='Brand Name')),
                ('missive_type', models.CharField(choices=[('postal', 'Postal mail'), ('lre', 'Electronic registered letter'), ('ere', 'Electronic registered email (ERE)'), ('email', 'Email'), ('email_marketing', 'Marketing email'), ('sms', 'SMS'), ('rcs', 'RCS (Rich SMS)'), ('voice_call', 'Automated voice call'), ('notification', 'In-app notification'), ('push_notification', 'Mobile push notification'), ('branded', 'Branded App messaging (WhatsApp, Slack, etc.)')], help_text='Type of missive (email, SMS, postal, etc.)', max_length=50, verbose_name='Missive Type')),
                ('acknowledgement', models.CharField(blank=True, choices=[('basic_delivery', 'Basic delivery'), ('acknowledgement_of_receipt', 'Acknowledgement of receipt')], help_text='Desired acknowledgement level for delivery proof', max_length=50, null=True, verbose_name='Acknowledgement Level')),
                ('delivery_mode', models.CharField(blank=True, choices=[('economic', 'Economic'), ('normal', 'Normal'), ('premium', 'Premium'), ('express', 'Express')], help_text='Delivery mode (email, SMS, postal, etc.)', max_length=50, null=True, verbose_name='Delivery Mode')),
                ('priority', models.CharField(blank=True, choices=[('low', 'Low'), ('normal', 'Normal'), ('high', 'High'), ('urgent', 'Urgent')], help_text='Priority level', max_length=20, null=True, verbose_name='Priority')),
                ('subject', models.CharField(blank=True, help_text='Subject line (for email, SMS, etc.)', max_length=500, null=True, verbose_name='Subject')),
                ('body_html', django_pymissive.fields.RichTextField(blank=True, help_text='HTML message body (email) or rich content (postal)', null=True, verbose_name='Body HTML')),
                ('body_text', models.TextField(blank=True, help_text='Plain text version of the message', null=True, verbose_name='Body Text')),
                ('sender_name', models.CharField(blank=True, help_text='Display name of the sender', max_length=255, null=True, verbose_name='Sender name')),
                ('sender_email', models.EmailField(blank=True, help_text='Email address of the sender', max_length=254, null=True, verbose_name='Sender email')),
                ('sender_phone', phonenumber_field.modelfields.PhoneNumberField(blank=True, help_text='Phone number of the sender (used for SMS)', max_length=128, null=True, region=None, verbose_name='Sender phone')),
                ('sender_address', django_geoaddress.fields.GeoaddressField(blank=True, help_text='Postal address of the sender', null=True, verbose_name='Sender address')),
                ('reply_to_name', models.CharField(blank=True, help_text='Display name for reply-to address', max_length=255, null=True, verbose_name='Reply-To name')),
                ('reply_to_email', models.EmailField(blank=True, help_text='Email address for replies', max_length=254, null=True, verbose_name='Reply-To email')),
                ('reply_to_address', django_geoaddress.fields.GeoaddressField(blank=True, help_text='Postal address for replies', max_length=512, null=True, verbose_name='Reply-To address')),
                ('additional_context', django_pymissive.fields.JSONField(blank=True, default=dict, help_text='Additional context as JSON', verbose_name='Additional context')),
                ('external_id', models.CharField(blank=True, editable=False, help_text='External identifier from the provider', max_length=255, null=True, verbose_name='External ID')),
                ('metadata', django_pymissive.fields.JSONField(blank=True, default=dict, help_text='Additional metadata as JSON', verbose_name='Metadata')),
                ('additional_config', django_pymissive.fields.JSONField(blank=True, default=dict, help_text='Additional configuration as JSON', verbose_name='Additional configuration')),
                ('webhook_url', models.URLField(blank=True, help_text='Webhook URL for the missive', max_length=255, null=True, verbose_name='Webhook URL')),
            ],
            options={
                'verbose_name': 'Missive',
                'verbose_name_plural': 'Missives',
                'ordering': ['-created_at'],
            },
        ),
        migrations.CreateModel(
            name='MissiveCampaign',
            fields=[
                ('comment', models.TextField(blank=True, help_text='Internal comment or note', null=True, verbose_name='Comment')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created At')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated At')),
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False, verbose_name='ID')),
                ('subject', models.CharField(help_text='Campaign subject', max_length=255, verbose_name='Subject')),
                ('description', django_pymissive.fields.RichTextField(blank=True, help_text='Campaign description', verbose_name='Description')),
                ('acknowledgement_email', models.CharField(choices=[('basic_delivery', 'Basic delivery'), ('acknowledgement_of_receipt', 'Acknowledgement of receipt')], default='basic_delivery', help_text='Desired acknowledgement level for delivery proof', max_length=50, verbose_name='Acknowledgement Level')),
                ('sender_email_name', models.CharField(blank=True, help_text='Campaign sender email name', max_length=255, null=True, verbose_name='Sender email name')),
                ('sender_email', models.EmailField(blank=True, help_text='Campaign sender email', max_length=254, null=True, verbose_name='Sender email')),
                ('reply_to_email_name', models.CharField(blank=True, help_text='Display name for reply-to address', max_length=255, null=True, verbose_name='Reply-To name')),
                ('reply_to_email', models.EmailField(blank=True, help_text='Email address for replies', max_length=254, null=True, verbose_name='Reply-To email')),
                ('body_html', django_pymissive.fields.RichTextField(blank=True, help_text='Campaign email HTML body', verbose_name='Body HTML')),
                ('body_text', models.TextField(blank=True, help_text='Campaign body text', verbose_name='Body text')),
                ('sender_phone_name', models.CharField(blank=True, help_text='Campaign sender phone name', max_length=255, null=True, verbose_name='Sender phone name')),
                ('sender_phone', phonenumber_field.modelfields.PhoneNumberField(blank=True, help_text='Phone number of the sender (used for SMS)', max_length=128, null=True, region=None, verbose_name='Sender phone')),
                ('body_sms', models.TextField(blank=True, help_text='Campaign body SMS', verbose_name='Body SMS')),
                ('sender_address_name', models.CharField(blank=True, help_text='Campaign sender address name', max_length=255, null=True, verbose_name='Sender address name')),
                ('sender_address', django_geoaddress.fields.GeoaddressField(blank=True, help_text='Campaign sender address', null=True, verbose_name='Sender address')),
                ('reply_to_address_name', models.CharField(blank=True, help_text='Display name for reply-to address', max_length=255, null=True, verbose_name='Reply-To address name')),
                ('reply_to_address', django_geoaddress.fields.GeoaddressField(blank=True, help_text='Postal address for replies', max_length=512, null=True, verbose_name='Reply-To address')),
                ('acknowledgement_postal', models.CharField(choices=[('basic_delivery', 'Basic delivery'), ('acknowledgement_of_receipt', 'Acknowledgement of receipt')], default='basic_delivery', help_text='Desired acknowledgement level for delivery proof', max_length=50, verbose_name='Acknowledgement Level')),
                ('delivery_mode_postal', models.CharField(choices=[('economic', 'Economic'), ('normal', 'Normal'), ('premium', 'Premium'), ('express', 'Express')], default='normal', help_text='Delivery mode (email, SMS, postal, etc.)', max_length=50, verbose_name='Delivery Mode')),
                ('priority_postal', models.CharField(choices=[('low', 'Low'), ('normal', 'Normal'), ('high', 'High'), ('urgent', 'Urgent')], default='normal', help_text='Priority level', max_length=20, verbose_name='Priority')),
                ('body_postal', django_pymissive.fields.RichTextField(blank=True, help_text='Campaign body Postal', verbose_name='Body Postal')),
                ('additional_context', django_pymissive.fields.JSONField(blank=True, default=dict, help_text='Additional context as JSON', verbose_name='Additional context')),
                ('additional_config', django_pymissive.fields.JSONField(blank=True, default=dict, help_text='Additional configuration as JSON', verbose_name='Additional configuration')),
                ('metadata', django_pymissive.fields.JSONField(blank=True, default=dict, help_text='Additional metadata as JSON', verbose_name='Metadata')),
            ],
            options={
                'verbose_name': 'Campaign',
                'verbose_name_plural': 'Campaigns',
                'ordering': [],
            },
        ),
        migrations.CreateModel(
            name='MissiveConfig',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('missive_type', models.CharField(choices=[('postal', 'Postal mail'), ('lre', 'Electronic registered letter'), ('ere', 'Electronic registered email (ERE)'), ('email', 'Email'), ('email_marketing', 'Marketing email'), ('sms', 'SMS'), ('rcs', 'RCS (Rich SMS)'), ('voice_call', 'Automated voice call'), ('notification', 'In-app notification'), ('push_notification', 'Mobile push notification'), ('branded', 'Branded App messaging (WhatsApp, Slack, etc.)')], help_text='Type of missive (email, sms, postal, etc.)', max_length=50, unique=True, verbose_name='Missive type')),
                ('default_provider', django_providerkit.fields.ProviderField(blank=True, help_text='Provider name (e.g. brevo, sendgrid). Leave empty for no default.', max_length=100, verbose_name='Default provider')),
            ],
            options={
                'verbose_name': 'Configuration',
                'verbose_name_plural': 'Configurations',
                'ordering': ['missive_type'],
            },
        ),
        migrations.CreateModel(
            name='MissiveHistory',
            fields=[
            ],
            options={
                'verbose_name': 'Missive History',
                'verbose_name_plural': 'Missive Histories',
                'ordering': ['-created_at'],
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('django_pymissive.missive',),
        ),
        migrations.CreateModel(
            name='MissiveMessage',
            fields=[
            ],
            options={
                'verbose_name': 'Missive Message',
                'verbose_name_plural': 'Missive Messages',
                'ordering': ['-created_at'],
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('django_pymissive.missive',),
        ),
        migrations.CreateModel(
            name='MissiveBaseAttachment',
            fields=[
                ('comment', models.TextField(blank=True, help_text='Internal comment or note', null=True, verbose_name='Comment')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created At')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated At')),
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False, verbose_name='ID')),
                ('attachment_type', models.CharField(choices=[('virtual_attachment', 'Virtual Attachment'), ('attachment', 'Attachment'), ('signature', 'Signature'), ('receipt', 'Receipt'), ('proof', 'Proof'), ('other', 'Other')], default='attachment', help_text='Type of attachment (attachment, signature, receipt, proof, other)', max_length=50, verbose_name='Attachment Type')),
                ('attachment_object_id', models.PositiveIntegerField(blank=True, help_text='ID of the object to which this file is attached', null=True, verbose_name='Object ID')),
                ('attachment_object_arguments', django_pymissive.fields.JSONField(blank=True, default={'args': [], 'kwargs': {}, 'method': 'get_attachment'}, help_text='Arguments to pass to the file method (as dict for **kwargs)', verbose_name='Attachment Object Arguments')),
                ('attachment_file', models.FileField(blank=True, help_text='Leave blank if the attachment is hosted externally', null=True, upload_to=django_pymissive.models.attachment._attachment_upload_to, verbose_name='Attachment File')),
                ('metadata', django_pymissive.fields.JSONField(blank=True, default=dict, help_text='Additional metadata as JSON', verbose_name='Metadata')),
                ('linked', models.BooleanField(default=True, help_text='Indicates if the attachment is linked to a related object', verbose_name='Linked')),
                ('priority', models.PositiveIntegerField(default=1, help_text='Page order (1=first page from letter body, attachments start at 2)', verbose_name='Priority')),
                ('external_id', models.CharField(blank=True, help_text='External ID of the attachment', max_length=255, null=True, verbose_name='External ID')),
                ('page_count', models.PositiveIntegerField(blank=True, help_text='Number of pages in the document', null=True, verbose_name='Page count')),
                ('attachment_content_type', models.ForeignKey(blank=True, help_text='Type of model to which this file is attached', null=True, on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype', verbose_name='Object Type')),
                ('missive', models.ForeignKey(blank=True, help_text='Missive to which this file is attached', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='to_missiveattachment', to='django_pymissive.missive', verbose_name='Missive')),
                ('campaign', models.ForeignKey(blank=True, help_text='Campaign to which this file is attached', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='to_campaigndocument', to='django_pymissive.missivecampaign', verbose_name='Campaign')),
            ],
            options={
                'verbose_name': 'Attachment',
                'verbose_name_plural': 'Attachments',
                'ordering': ['priority'],
            },
        ),
        migrations.CreateModel(
            name='CampaignAttachment',
            fields=[
            ],
            options={
                'verbose_name': 'Campaign Attachment',
                'verbose_name_plural': 'Campaign Attachments',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('django_pymissive.missivebaseattachment',),
        ),
        migrations.CreateModel(
            name='CampaignVirtualAttachment',
            fields=[
            ],
            options={
                'verbose_name': 'Campaign Virtual Attachment',
                'verbose_name_plural': 'Campaign Virtual Attachments',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('django_pymissive.missivebaseattachment',),
        ),
        migrations.CreateModel(
            name='MissiveAttachment',
            fields=[
            ],
            options={
                'verbose_name': 'Attachment',
                'verbose_name_plural': 'Attachments',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('django_pymissive.missivebaseattachment',),
        ),
        migrations.CreateModel(
            name='MissiveVirtualAttachment',
            fields=[
            ],
            options={
                'verbose_name': 'Virtual Attachment',
                'verbose_name_plural': 'Virtual Attachments',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('django_pymissive.missivebaseattachment',),
        ),
        migrations.AddField(
            model_name='missive',
            name='campaign',
            field=models.ForeignKey(blank=True, help_text='Optional campaign this missive belongs to', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='to_missive', to='django_pymissive.missivecampaign', verbose_name='Campaign'),
        ),
        migrations.CreateModel(
            name='CampaignRelatedObject',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('comment', models.TextField(blank=True, help_text='Internal comment or note', null=True, verbose_name='Comment')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created At')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated At')),
                ('object_id', models.PositiveIntegerField(help_text='ID of the related object', verbose_name='Object ID')),
                ('object_str', models.CharField(editable=False, help_text='String representation of the related object (saved for reference if object is deleted)', max_length=500, verbose_name='Object String Representation')),
                ('content_type', models.ForeignKey(help_text='Type of the related object', on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype', verbose_name='Content Type')),
                ('campaign', models.ForeignKey(help_text='Campaign to which this object is related', on_delete=django.db.models.deletion.CASCADE, related_name='to_campaignrelatedobject', to='django_pymissive.missivecampaign', verbose_name='Campaign')),
            ],
            options={
                'verbose_name': 'Campaign Related Object',
                'verbose_name_plural': 'Campaign Related Objects',
                'ordering': ['-created_at'],
            },
        ),
        migrations.CreateModel(
            name='MissiveRecipient',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('comment', models.TextField(blank=True, help_text='Internal comment or note', null=True, verbose_name='Comment')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created At')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated At')),
                ('recipient_support', models.CharField(blank=True, choices=[('email', 'Email'), ('phone', 'Phone'), ('address', 'Address'), ('notification', 'Notification'), ('branded', 'Branded')], help_text='Model of recipient', max_length=255, null=True, verbose_name='Recipient Model')),
                ('recipient_type', models.CharField(choices=[('recipient', 'Recipient'), ('cc', 'CC'), ('bcc', 'BCC')], default='recipient', help_text='Type of recipient', max_length=20, verbose_name='Recipient Type')),
                ('status', models.CharField(choices=[('draft', 'Draft'), ('processing', 'Processing'), ('success', 'Success'), ('failed', 'Failed'), ('error', 'Error'), ('cancelled', 'Cancelled')], default='draft', help_text='Current status of the missive', max_length=20, verbose_name='Status')),
                ('name', models.CharField(blank=True, help_text='Full name or company name', max_length=255, verbose_name='Name')),
                ('email', models.EmailField(blank=True, help_text="Recipient's email address", max_length=254, null=True, verbose_name='Recipient Email')),
                ('phone', phonenumber_field.modelfields.PhoneNumberField(blank=True, help_text='Phone number', max_length=20, null=True, region=None, verbose_name='Phone')),
                ('address', django_geoaddress.fields.GeoaddressField(blank=True, help_text='Address', null=True, verbose_name='Address')),
                ('notification_id', models.CharField(blank=True, help_text='Notification ID', max_length=255, null=True, verbose_name='Notification ID')),
                ('external_id', models.CharField(blank=True, help_text='External identifier from the provider', max_length=255, null=True, verbose_name='External ID')),
                ('sent_at', models.DateTimeField(blank=True, help_text='When the missive was sent', null=True, verbose_name='Sent At')),
                ('delivered_at', models.DateTimeField(blank=True, help_text='When the missive was delivered', null=True, verbose_name='Delivered At')),
                ('missive', models.ForeignKey(help_text='Missive', on_delete=django.db.models.deletion.CASCADE, related_name='to_missiverecipient', to='django_pymissive.missive', verbose_name='Missive')),
            ],
            options={
                'verbose_name': 'Recipient',
                'verbose_name_plural': 'Recipients',
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='MissiveEvent',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('comment', models.TextField(blank=True, help_text='Internal comment or note', null=True, verbose_name='Comment')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created At')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated At')),
                ('provider', django_providerkit.fields.ProviderField(blank=True, help_text='Provider that emitted this event', max_length=100, null=True, verbose_name='Provider')),
                ('event', models.CharField(blank=True, choices=[('delivered', 'Delivered'), ('read', 'Read'), ('opened', 'Opened'), ('click', 'Clicked'), ('clicked', 'Clicked'), ('proofs_of_delivery', 'Proofs of Delivery'), ('untreated', 'Untreated'), ('draft', 'Draft'), ('sent', 'Sent'), ('prepare', 'Prepare'), ('pending', 'Pending'), ('processing', 'Processing'), ('queued', 'Queued'), ('request', 'Request'), ('deferred', 'Deferred'), ('scheduled', 'Scheduled'), ('unknown_type', 'Unknown type'), ('attempted_delivery', 'Attempted delivery'), ('mailed', 'Mailed'), ('printed', 'Printed'), ('accepted', 'Accepted'), ('dispatched', 'Dispatched'), ('shipped', 'Shipped'), ('in_transit', 'In transit'), ('deposit_proof', 'Deposit Proof'), ('proof_of_content', 'Proof of Content'), ('archived', 'Archived'), ('failed', 'Failed'), ('cancelled', 'Cancelled'), ('bounced', 'Bounced'), ('hard_bounce', 'Hard bounce'), ('soft_bounce', 'Soft bounce'), ('dropped', 'Dropped'), ('spam', 'Spam'), ('blocked', 'Blocked'), ('invalid', 'Invalid'), ('mailbox_not_found', 'Mailbox not found'), ('blocklisted', 'Blocklisted'), ('blocklist_created', 'Blocklist created'), ('rejected', 'Rejected'), ('undelivered', 'Undelivered'), ('undeliverable', 'Undeliverable'), ('expired', 'Expired'), ('carrier_rejected', 'Carrier rejected'), ('return_to_sender', 'Return to sender'), ('refused', 'Refused'), ('lost', 'Lost'), ('damaged', 'Damaged'), ('out_for_delivery', 'Out for delivery'), ('error', 'Error')], help_text='Event type (sent, delivered, read, failed, etc.)', max_length=50, null=True, verbose_name='Event')),
                ('description', models.TextField(blank=True, help_text='Description or details about this event', verbose_name='Description')),
                ('metadata', django_pymissive.fields.JSONField(blank=True, default=dict, help_text='Additional metadata as JSON', verbose_name='Metadata')),
                ('trace', django_pymissive.fields.JSONField(blank=True, default=dict, help_text='Raw trace data (webhook payload, API response, etc.)', verbose_name='Trace')),
                ('user_action', models.BooleanField(default=False, help_text='Indicates if the event was triggered by a user action', verbose_name='User Action')),
                ('occurred_at', models.DateTimeField(help_text='When this event occurred', verbose_name='Occurred At')),
                ('is_billed', models.BooleanField(default=False, help_text='Indicates if the missive has been billed', verbose_name='Billed')),
                ('billing_amount', models.DecimalField(blank=True, decimal_places=3, help_text='Amount billed for the missive', max_digits=10, null=True, verbose_name='Billing Amount')),
                ('estimate_amount', models.DecimalField(blank=True, decimal_places=3, help_text='Estimated amount for the missive', max_digits=10, null=True, verbose_name='Estimate Amount')),
                ('missive', models.ForeignKey(blank=True, editable=False, help_text='Missive associated with this event', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='to_missiveevent', to='django_pymissive.missive', verbose_name='Missive')),
                ('recipient', models.ForeignKey(blank=True, editable=False, help_text='Recipient associated with this event', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='to_recipientevent', to='django_pymissive.missiverecipient', verbose_name='Recipient')),
            ],
            options={
                'verbose_name': 'Event',
                'verbose_name_plural': 'Events',
                'ordering': ['-occurred_at'],
            },
        ),
        migrations.AddField(
            model_name='missive',
            name='message_by',
            field=models.ForeignKey(blank=True, help_text='Recipient who sent this reply (for inbound exchanges)', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='to_missivemessageby', to='django_pymissive.missiverecipient', verbose_name='Reply by'),
        ),
        migrations.CreateModel(
            name='MissiveRecipientAddress',
            fields=[
            ],
            options={
                'verbose_name': 'Address Recipient',
                'verbose_name_plural': 'Address Recipients',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('django_pymissive.missiverecipient',),
        ),
        migrations.CreateModel(
            name='MissiveRecipientEmail',
            fields=[
            ],
            options={
                'verbose_name': 'Email Recipient',
                'verbose_name_plural': 'Email Recipients',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('django_pymissive.missiverecipient',),
        ),
        migrations.CreateModel(
            name='MissiveRecipientNotification',
            fields=[
            ],
            options={
                'verbose_name': 'Notification Recipient',
                'verbose_name_plural': 'Notification Recipients',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('django_pymissive.missiverecipient',),
        ),
        migrations.CreateModel(
            name='MissiveRecipientPhone',
            fields=[
            ],
            options={
                'verbose_name': 'Phone Recipient',
                'verbose_name_plural': 'Phone Recipients',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('django_pymissive.missiverecipient',),
        ),
        migrations.CreateModel(
            name='MissiveRelatedObject',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('comment', models.TextField(blank=True, help_text='Internal comment or note', null=True, verbose_name='Comment')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created At')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated At')),
                ('object_id', models.PositiveIntegerField(help_text='ID of the related object', verbose_name='Object ID')),
                ('object_str', models.CharField(editable=False, help_text='String representation of the related object (saved for reference if object is deleted)', max_length=500, verbose_name='Object String Representation')),
                ('content_type', models.ForeignKey(help_text='Type of the related object', on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype', verbose_name='Content Type')),
                ('missive', models.ForeignKey(help_text='Missive to which this object is related', on_delete=django.db.models.deletion.CASCADE, related_name='to_missiverelatedobject', to='django_pymissive.missive', verbose_name='Missive')),
            ],
            options={
                'verbose_name': 'Missive Related Object',
                'verbose_name_plural': 'Missive Related Objects',
                'ordering': ['-created_at'],
            },
        ),
        migrations.CreateModel(
            name='MissiveScheduledCampaign',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('comment', models.TextField(blank=True, help_text='Internal comment or note', null=True, verbose_name='Comment')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created At')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated At')),
                ('scheduled_send_date', models.DateTimeField(blank=True, help_text='Scheduled send date for the campaign (leave blank for immediate sending)', null=True, verbose_name='Scheduled send date')),
                ('send_date', models.DateTimeField(blank=True, editable=False, help_text='Actual send date for the campaign', null=True, verbose_name='Send date')),
                ('ended_at', models.DateTimeField(blank=True, editable=False, help_text='Actual ended date for the campaign', null=True, verbose_name='Ended at')),
                ('additional_config', django_pymissive.fields.JSONField(blank=True, default=dict, help_text='Additional configuration as JSON', verbose_name='Additional configuration')),
                ('campaign', models.ForeignKey(editable=False, on_delete=django.db.models.deletion.CASCADE, related_name='to_missivecampaignsend', to='django_pymissive.missivecampaign', verbose_name='Campaign')),
            ],
            options={
                'verbose_name': 'Campaign send',
                'verbose_name_plural': 'Campaign sends',
                'ordering': ['-scheduled_send_date', '-ended_at', '-id'],
            },
        ),
    ]



"""
Generate translation_catalog.py from pymissive.config.

Strings in pymissive.config (pymissive/config/__init__.py) are used via variables in
django_pymissive.models.choices, so xgettext (makemessages) cannot extract them.
This command generates a Python module with explicit _("literal") calls so makemessages
can find them.

Sources: MISSIVE_ACKNOWLEDGEMENT_LEVELS, MISSIVE_TYPES, SUCCESSFUL_EVENTS, INFO_EVENTS,
FAILED_EVENTS, DELIVERY_MODES, PRIORITIES, GENERIC_SUPPORT.

Run before makemessages:
  ./manage.py update_translation_catalog
  ./manage.py makemessages -l fr
"""

from pathlib import Path

from django.core.management.base import BaseCommand


def extract_strings_from_config() -> set[str]:
    """Extract all user-facing strings from pymissive.config."""
    from pymissive.config import (
        MISSIVE_ACKNOWLEDGEMENT_LEVELS,
        MISSIVE_TYPES,
        SUCCESSFUL_EVENTS,
        INFO_EVENTS,
        FAILED_EVENTS,
        DELIVERY_MODES,
        PRIORITIES,
        GENERIC_SUPPORT,
    )

    strings: set[str] = set()

    for level in MISSIVE_ACKNOWLEDGEMENT_LEVELS:
        strings.add(level["display_name"])
        if level.get("description"):
            strings.add(level["description"])
        if level.get("legal_value"):
            strings.add(level["legal_value"])
        for m in level.get("means") or []:
            strings.add(m)

    for v in MISSIVE_TYPES.values():
        strings.add(v)

    def _add_event_strings(events_dict):
        for v in events_dict.values():
            if isinstance(v, tuple):
                strings.add(v[0])
                if len(v) > 1 and v[1]:
                    strings.add(v[1])
            else:
                strings.add(v)

    _add_event_strings(SUCCESSFUL_EVENTS)
    _add_event_strings(INFO_EVENTS)
    _add_event_strings(FAILED_EVENTS)

    for k in DELIVERY_MODES:
        strings.add(k.capitalize())

    for k in PRIORITIES:
        strings.add(k.capitalize())

    for k in GENERIC_SUPPORT.keys():
        strings.add(k.capitalize())

    return strings


def generate_catalog_content(strings: set[str]) -> str:
    """Generate translation_catalog.py source."""
    sorted_strings = sorted(strings, key=lambda s: (s.lower(), s))

    lines = [
        '"""',
        "Translation catalog for pymissive.config strings.",
        "",
        "Auto-generated by: ./manage.py update_translation_catalog",
        "Do not edit manually. Run update_translation_catalog after config changes.",
        '"""',
        "",
        "from django.utils.translation import gettext_lazy as _",
        "",
        "# Strings from pymissive.config - used by makemessages for extraction",
        "",
    ]

    for s in sorted_strings:
        if not s or not isinstance(s, str):
            continue
        # Escape backslash and quote for Python string literal
        escaped = s.replace("\\", "\\\\").replace('"', '\\"')
        lines.append(f'_("{escaped}")')

    return "\n".join(lines) + "\n"


class Command(BaseCommand):
    help = "Generate translation_catalog.py from pymissive.config for makemessages extraction"

    def handle(self, *args, **options):
        strings = extract_strings_from_config()
        content = generate_catalog_content(strings)

        package_dir = Path(__file__).resolve().parent.parent.parent
        output_path = package_dir / "translation_catalog.py"

        output_path.write_text(content, encoding="utf-8")

        self.stdout.write(
            self.style.SUCCESS(
                f"Generated {output_path} with {len(strings)} translatable strings."
            )
        )
        self.stdout.write(
            "Run: ./manage.py makemessages -l fr"
        )



"""MissiveAttachment model."""

import os
import uuid
from datetime import date

from django.conf import settings
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.core.files.storage import Storage, default_storage
from django.db import models
from django.urls import reverse
from django.utils.deconstruct import deconstructible
from django.utils.module_loading import import_string
from django.utils.translation import gettext_lazy as _

from .base import CommentTimestampedModel
from .choices import MissiveAttachmentType
from ..utils import get_base_url
from ..managers.attachment import (
    MissiveBaseAttachmentManager,
    MissiveAttachmentManager,
    MissiveVirtualAttachmentManager,
    CampaignAttachmentManager,
    CampaignVirtualAttachmentManager,
    MissiveProofManager,
)
from ..fields import JSONField

# Priority 0 is reserved for the first-document (letter body PDF). Other attachments use 1, 2, 3...
FIRST_DOCUMENT_PRIORITY = 0


def _attachment_upload_to(instance, filename):
    """Return upload path based on attachment class."""
    upload_to = getattr(settings, 'PYMISSIVE_ATTACHMENT_UPLOAD_TO', None)
    if upload_to:
        return import_string(upload_to)(instance, filename)
    today = date.today()
    prefix = "campaign" if instance.campaign else "missive"
    attachment_type = instance.attachment_type.lower()
    uid = str(instance.pk)
    return f"pymissive/{prefix}/{attachment_type}/{uid}/{today:%Y/%m/%d}/{filename}"


def _get_attachment_file_storage():
    """Return storage for attachment_file. Configure via PYMISSIVE_ATTACHMENT_FILE_STORAGE.
    - None: use default_storage (MEDIA_ROOT)
    - str: import path to storage class, instantiated with ()
    - instance: use as-is (e.g. DataroomStorage())
    """
    storage = getattr(settings, 'PYMISSIVE_ATTACHMENT_FILE_STORAGE', None)
    if storage is None:
        return default_storage
    if isinstance(storage, str):
        storage_class = import_string(storage)
        return storage_class()
    return storage


@deconstructible
class ConfigurableAttachmentStorage(Storage):
    """Storage that delegates to PYMISSIVE_ATTACHMENT_FILE_STORAGE at runtime.
    Defined in django_pymissive to avoid migration dependency on project-specific storages.
    """

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self._storage = None

    @property
    def _backend(self):
        if self._storage is None:
            self._storage = _get_attachment_file_storage()
        return self._storage

    def _open(self, name, mode='rb'):
        return self._backend._open(name, mode)

    def _save(self, name, content, max_length=None):
        # Override via PYMISSIVE_ATTACHMENT_PATH_MAX_LENGTH; cannot exceed field max_length.
        effective = getattr(settings, "PYMISSIVE_ATTACHMENT_PATH_MAX_LENGTH", None)
        if effective is not None:
            max_length = min(max_length or 2000, effective)
        try:
            return self._backend._save(name, content, max_length=max_length)
        except TypeError:
            return self._backend._save(name, content)

    def exists(self, name):
        return self._backend.exists(name)

    def delete(self, name):
        return self._backend.delete(name)

    def url(self, name):
        return self._backend.url(name)

    def size(self, name):
        return self._backend.size(name)

    def __getattr__(self, name):
        return getattr(self._backend, name)


class MissiveBaseAttachment(CommentTimestampedModel):
    """File attachment for missives or any other model."""

    id = models.UUIDField(
        primary_key=True,
        default=uuid.uuid4,
        editable=False,
        verbose_name=_("ID"),
    )

    campaign = models.ForeignKey(
        "django_pymissive.MissiveCampaign",
        on_delete=models.CASCADE,
        related_name="to_campaigndocument",
        verbose_name=_("Campaign"),
        null=True,
        blank=True,
        help_text=_("Campaign to which this file is attached"),
    )
    
    missive = models.ForeignKey(
        "django_pymissive.Missive",
        on_delete=models.CASCADE,
        related_name="to_missiveattachment",
        verbose_name=_("Missive"),
        null=True,
        blank=True,
        help_text=_("Missive to which this file is attached"),
    )

    attachment_type = models.CharField(
        max_length=50,
        choices=MissiveAttachmentType.choices,
        default=MissiveAttachmentType.ATTACHMENT,
        verbose_name=_("Attachment Type"),
        help_text=_("Type of attachment (attachment, signature, receipt, proof, other)"),
    )

    attachment_content_type = models.ForeignKey(
        ContentType,
        on_delete=models.CASCADE,
        null=True,
        blank=True,
        verbose_name=_("Object Type"),
        help_text=_("Type of model to which this file is attached"),
    )

    attachment_object_id = models.PositiveIntegerField(
        null=True,
        blank=True,
        verbose_name=_("Object ID"),
        help_text=_("ID of the object to which this file is attached"),
    )

    attachment_object_arguments = JSONField(
        default=dict({"method": "retrieve_attachment", "args": [], "kwargs": {}}),
        blank=True,
        verbose_name=_("Attachment Object Arguments"),
        help_text=_("Arguments to pass to the file method (as dict for **kwargs)"),
    )

    attachment_file = models.FileField(
        upload_to=_attachment_upload_to,
        storage=ConfigurableAttachmentStorage(),
        max_length=2000,
        blank=True,
        null=True,
        verbose_name=_("Attachment File"),
        help_text=_("Leave blank if the attachment is hosted externally"),
    )

    metadata = JSONField(
        default=dict,
        blank=True,
        verbose_name=_("Metadata"),
        help_text=_("Additional metadata as JSON"),
    )

    linked = models.BooleanField(
        default=True,
        verbose_name=_("Linked"),
        help_text=_("Indicates if the attachment is linked to a related object"),
    )

    priority = models.PositiveIntegerField(
        default=1,
        verbose_name=_("Priority"),
        help_text=_("Page order. 0=first-document (letter body), others use 1, 2, 3... (0 reserved)"),
    )

    external_id = models.CharField(
        max_length=255,
        blank=True,
        null=True,
        verbose_name=_("External ID"),
        help_text=_("External ID of the attachment"),
    )

    page_count = models.PositiveIntegerField(
        null=True,
        blank=True,
        verbose_name=_("Page count"),
        help_text=_("Number of pages in the document"),
    )

    attachment_object = GenericForeignKey("attachment_content_type", "attachment_object_id")
    objects = MissiveBaseAttachmentManager()

    class Meta:
        verbose_name = _("Attachment")
        verbose_name_plural = _("Attachments")
        ordering = ["priority",]

    @property
    def is_first_document(self):
        """True if this is the first-document (letter body PDF). Reserved for priority 0."""
        name = getattr(self.attachment_file, "name", None) or ""
        return "first-document-" in name

    @property
    def can_be_modified(self):
        if hasattr(self, "missive"):
            return self.missive.can_be_modified
        if hasattr(self, "campaign"):
            return self.campaign.can_be_modified
        return False

    def can_access_document(self):
        """Checks if the document can be accessed."""
        return all(
            [
                self.attachment_content_type,
                self.attachment_object_id,
                self.attachment_object_arguments,
            ]
        )

    def get_virtual_attachment(self):
        """Gets the virtual attachment by calling the configured method on the related object."""
        get_method = self.attachment_object_arguments["method"]
        args = self.attachment_object_arguments.get("args", [])
        kwargs = self.attachment_object_arguments.get("kwargs", {})
        return getattr(self.attachment_object, get_method)(*args, **kwargs)

    def get_attachment(self):
        """Returns the raw file or virtual attachment object."""
        if self.attachment_type == MissiveAttachmentType.VIRTUAL_ATTACHMENT:
            return self.get_virtual_attachment()
        return self.attachment_file

    @property
    def attachment_url(self):
        return self.get_serialized_attachment(linked=True, ignore_content=True)

    def get_serialized_attachment(self, linked=False, ignore_content=False):
        """Returns a serialized dict for this attachment."""
        attachment = self.get_attachment()
        name = getattr(attachment, "name", None) or "unnamed_attachment"
        scope = "campaign" if self.campaign_id else "missive"
        url = reverse("django_pymissive:missive_attachment_download", args=[scope, self.id])
        base_url = get_base_url(trailing_slash=False)
        data = {
            "id": str(self.id),
            "external_id": self.external_id,
            "priority": self.priority,
            "name": os.path.basename(name),
            "url": base_url + url,
        }
        if linked or ignore_content:
            return data
        if hasattr(attachment, "seek"):
            attachment.seek(0)
        data["content"] = attachment.read()
        return data

    def calculate_priority(self):
        """Return next priority. First-document uses 0; others use 1, 2, 3... (0 is reserved)."""
        from django.db.models import Max

        if self.is_first_document:
            return FIRST_DOCUMENT_PRIORITY
        qs = MissiveBaseAttachment.objects
        if self.missive_id:
            qs = qs.filter(missive_id=self.missive_id)
        elif self.campaign_id:
            qs = qs.filter(campaign_id=self.campaign_id)
        else:
            return 1
        # Exclude first-documents (priority 0) from max; others start at 1
        qs = qs.exclude(priority=FIRST_DOCUMENT_PRIORITY)
        max_priority = qs.aggregate(Max("priority"))["priority__max"] or (FIRST_DOCUMENT_PRIORITY)
        return max(1, max_priority + 1)

    def _recalculate_sibling_priorities(self):
        """Reassign sequential priorities (1, 2, 3...) when one attachment's priority changed."""
        from ..utils import recalculate_attachment_priorities

        recalculate_attachment_priorities(missive_id=self.missive_id, campaign_id=self.campaign_id)

    def can_access_attachment(self):
        """Checks if the attachment can be accessed."""
        if self.attachment_type == MissiveAttachmentType.VIRTUAL_ATTACHMENT:
            return (self.attachment_object and self.attachment_object_arguments)
        return self.attachment_file and self.attachment_file.url

    def save(self, *args, **kwargs):
        """Auto-set priority for new attachments. First-document=0, others 1+. Recalc in admin save_formset."""
        if not self.pk and (self.missive_id or self.campaign_id):
            self.priority = self.calculate_priority()
        else:
            if self.is_first_document:
                self.priority = FIRST_DOCUMENT_PRIORITY
            elif self.priority == FIRST_DOCUMENT_PRIORITY:
                self.priority = 1
        super().save(*args, **kwargs)

class MissiveAttachment(MissiveBaseAttachment):
    """Attachment for missives."""

    objects = MissiveAttachmentManager()

    class Meta:
        proxy = True
        verbose_name = _("Attachment")
        verbose_name_plural = _("Attachments")


class MissiveVirtualAttachment(MissiveBaseAttachment):
    """Virtual attachment for missives."""

    objects = MissiveVirtualAttachmentManager()

    class Meta:
        proxy = True
        verbose_name = _("Virtual Attachment")
        verbose_name_plural = _("Virtual Attachments")


class CampaignAttachment(MissiveBaseAttachment):
    """Attachment for campaigns."""

    objects = CampaignAttachmentManager()

    class Meta:
        proxy = True
        verbose_name = _("Campaign Attachment")
        verbose_name_plural = _("Campaign Attachments")


class CampaignVirtualAttachment(MissiveBaseAttachment):
    """Virtual attachment for campaigns."""

    objects = CampaignVirtualAttachmentManager()

    class Meta:
        proxy = True
        verbose_name = _("Campaign Virtual Attachment")
        verbose_name_plural = _("Campaign Virtual Attachments")

class MissiveProof(MissiveBaseAttachment):
    """Proof for missives."""

    objects = MissiveProofManager()

    class Meta:
        proxy = True
        verbose_name = _("Proof")
        verbose_name_plural = _("Proofs")


"""Utility functions for django_pymissive."""

from urllib.parse import urlparse

from django.conf import settings
from django.urls import reverse

from .models.recipient import MissiveRecipient


def recalculate_attachment_priorities(missive_id=None, campaign_id=None):
    """
    Reassign priorities: first-document stays at 0, others get 1, 2, 3...
    Use after inline save or when priority changed programmatically.
    """
    if not missive_id and not campaign_id:
        return
    from .models.attachment import FIRST_DOCUMENT_PRIORITY, MissiveBaseAttachment

    qs = MissiveBaseAttachment.objects
    if missive_id:
        qs = qs.filter(missive_id=missive_id)
    else:
        qs = qs.filter(campaign_id=campaign_id)
    siblings = list(qs.order_by("priority", "id"))
    to_update = []
    next_priority = FIRST_DOCUMENT_PRIORITY
    for att in siblings:
        if att.is_first_document:
            expected = FIRST_DOCUMENT_PRIORITY
        else:
            next_priority = max(next_priority + 1, 1)
            expected = next_priority
        if att.priority != expected:
            att.priority = expected
            to_update.append(att)
    if to_update:
        MissiveBaseAttachment.objects.bulk_update(to_update, ["priority"])


def get_default_domain():
    """Return default domain (host) from settings or localhost:8000.
    If setting is a full URL, extracts the netloc for use in webhook domain field.
    """
    domain = (
        getattr(settings, "MISSIVE_DOMAIN", None)
        or getattr(settings, "DOMAIN", None)
        or "localhost:8000"
    )
    domain = str(domain).strip().rstrip("/")
    if domain.startswith(("http://", "https://")):
        parsed = urlparse(domain)
        return parsed.netloc or domain
    return domain


def get_default_scheme():
    """Return default scheme from settings (MISSIVE_SCHEME, SCHEME) or http.
    If domain setting is a full URL, extracts scheme from it.
    """
    scheme = getattr(settings, "MISSIVE_SCHEME", None) or getattr(settings, "SCHEME", None)
    if scheme:
        return str(scheme).replace("://", "")
    domain = getattr(settings, "MISSIVE_DOMAIN", None) or getattr(settings, "DOMAIN", None)
    if domain and str(domain).strip().lower().startswith("https://"):
        return "https"
    return "http"


def get_base_url(domain=None, scheme=None, trailing_slash=True):
    """
    Build base URL from domain and scheme.
    Defaults: domain=localhost:8000, scheme=http.
    Returns e.g. http://localhost:8000/ (with trailing_slash) or http://localhost:8000.
    """
    domain = domain or get_default_domain()
    scheme = scheme or get_default_scheme()
    scheme = str(scheme).replace("://", "")
    domain = str(domain).strip().rstrip("/")
    if domain.startswith(("http://", "https://")):
        base = domain.rstrip("/")
    else:
        base = f"{scheme}://{domain}"
    return f"{base}/" if trailing_slash else base


def build_webhook_url(domain: str, provider_name: str, missive_type: str) -> str:
    """Build full webhook URL from domain, provider and missive type."""
    domain = (domain or "").rstrip("/")
    path = reverse(
        "django_pymissive:missive_webhook",
        kwargs={"provider": provider_name, "missive_type": missive_type},
    )
    return f"{domain}{path}"


def get_recipient(missive, recipient_data):
    """Resolve recipient from missive and recipient_data dict."""
    if not isinstance(recipient_data, dict):
        return None
    try:
        return MissiveRecipient.objects.get(missive=missive, **recipient_data)
    except MissiveRecipient.DoesNotExist:
        return None


# Generated by Django 6.0.2 on 2026-03-12 23:52

import django_pymissive.models.attachment
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("django_pymissive", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="missivebaseattachment",
            name="attachment_file",
            field=models.FileField(
                blank=True,
                help_text="Leave blank if the attachment is hosted externally",
                null=True,
                storage=django_pymissive.models.attachment.ConfigurableAttachmentStorage(),
                upload_to=django_pymissive.models.attachment._attachment_upload_to,
                verbose_name="Attachment File",
            ),
        ),
    ]



"""Django Missive - Django library for missive management."""

from importlib.metadata import PackageNotFoundError, version

try:
    __version__ = version("django-pymissive")
except PackageNotFoundError:
    __version__ = "0.0.0.dev"



"""Custom dispatch signals for Missive (subscribe via django.dispatch.receiver).

Duplicate (see Missive.duplicate_missive):
- missive_pre_duplicate: source_missive, resend, thread_type, thread_id
- missive_post_duplicate: source_missive, new_missive, resend

Send (see Missive.send_missive):
- missive_pre_send / missive_post_send: missive (the row being sent), old_missive
  (None for a first send; the previous HISTORY missive when sending a resend duplicate).
"""

from django.dispatch import Signal

missive_pre_duplicate = Signal()
missive_post_duplicate = Signal()

missive_pre_send = Signal()
missive_post_send = Signal()



"""Provider model for missive providers."""

from django.db import models
from django.utils.translation import gettext_lazy as _
from virtualqueryset.models import VirtualModel
from django_providerkit.models.define import define_provider_fields, define_service_fields
from django_providerkit.managers import BaseProviderManager
from pymissive.providers.base import MissiveProviderBase


services = list(MissiveProviderBase.services_cfg.keys())


@define_provider_fields(primary_key="name")
@define_service_fields(services)
class MissiveProviderModel(VirtualModel):
    """Virtual model for missive providers."""

    name: models.CharField = models.CharField(
        max_length=255,
        verbose_name=_("Name"),
        help_text=_("Provider name (e.g., sendgrid)"),
        primary_key=True,
    )

    objects = BaseProviderManager(package_name='pymissive')

    class Meta:
        managed = False
        app_label = "django_pymissive"
        verbose_name = _("Provider")
        verbose_name_plural = _("Providers")
        ordering = ["-priority", "name"]

    def __str__(self) -> str:
        display = getattr(self, "display_name", None)
        return str(display or self.name)



"""Admin for MissiveCampaign model."""

from urllib.parse import unquote

from django.contrib import admin
from django.urls import reverse
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _
from django.utils.text import format_lazy
from django.contrib import messages
from django.shortcuts import redirect
from phonenumber_field.modelfields import PhoneNumberField
from phonenumber_field.formfields import PhoneNumberField as PhoneNumberFormField
from phonenumber_field.formfields import SplitPhoneNumberField
from django_boosted import AdminBoostModel
from django_boosted.decorators import admin_boost_view, admin_boost_action

from pymissive.config import MISSIVE_TYPES

from ..models.campaign import MissiveCampaign, MissiveScheduledCampaign
from ..models.attachment import MissiveBaseAttachment
from ..utils import recalculate_attachment_priorities
from .attachment import CampaignAttachmentInline, CampaignVirtualAttachmentInline
from .related_object import CampaignRelatedObjectInline 


@admin.register(MissiveScheduledCampaign)
class MissiveScheduledCampaignAdmin(AdminBoostModel):
    """Admin for missive scheduled campaign model."""

    list_display = [
        "campaign",
        "scheduled_send_date",
        "send_date",
        "ended_at",
        "comment",
        "additional_config",
    ]
    readonly_fields = [
        "campaign",
        "scheduled_send_date",
        "send_date",
        "ended_at",
        "created_at",
        "updated_at",
    ]


class MissiveScheduledCampaignInline(admin.TabularInline):
    """Inline for missive scheduled campaign model."""

    model = MissiveScheduledCampaign
    extra = 0
    fields = [
        "scheduled_send_date",
        "additional_config",
        "send_date",
        "ended_at",
    ]
    readonly_fields = [
        "campaign",
        "send_date",
        "ended_at",
    ]

    def has_change_permission(self, request, obj=None):
        return False


@admin.register(MissiveCampaign)
class MissiveCampaignAdmin(AdminBoostModel):
    """Admin for missive campaign model."""

    list_display = [
        "subject_display",
        "types_display",
        "stats_display",
        "last_send_date_display",
        "last_ended_at_display",
    ]
    search_fields = ["subject"]
    ordering = ["-id"]
    readonly_fields = [
        "buttons_show_and_preview_email",
        "buttons_show_and_preview_sms",
        "buttons_show_and_preview_lre",
        "created_at",
        "updated_at",
    ]
    inlines = [
        MissiveScheduledCampaignInline,
        CampaignAttachmentInline,
        CampaignVirtualAttachmentInline,
        CampaignRelatedObjectInline,
    ]

    def save_formset(self, request, form, formset, change):
        super().save_formset(request, form, formset, change)
        if formset.model and issubclass(formset.model, MissiveBaseAttachment):
            self._recalculate_attachment_priorities(formset, form.instance)

    def _recalculate_attachment_priorities(self, formset, parent):
        """Recalculate attachment priorities after inline save (admin bypasses model save logic)."""
        recalculate_attachment_priorities(campaign_id=parent.pk if parent else None)

    def formfield_for_dbfield(self, db_field, request, **kwargs):
        if isinstance(db_field, PhoneNumberField):
            kwargs.setdefault("required", False)
            if db_field.null:
                return PhoneNumberFormField(**kwargs)
            return SplitPhoneNumberField(**kwargs)
        return super().formfield_for_dbfield(db_field, request, **kwargs)

    changeform_actions = {
        "send_campaign": _("Send Campaign"),
    }
    fieldsets = [
        (
            None,
            {
                "fields": (
                    "subject",
                    "description",
                )
            },
        ),
    ]

    def change_fieldsets(self):
        """Configure fieldsets for change view."""
        self.add_to_fieldset(
            _("Email"),
            [
                "sender_email_name",
                "sender_email",
                "acknowledgement_email",
                "reply_to_email_name",
                "reply_to_email",
                "body_text",
                "body_html",
                "buttons_show_and_preview_email",
            ],
        )
        self.add_to_fieldset(
            _("SMS"),
            [
                "sender_phone_name",
                "sender_phone",
                "body_sms",
                "buttons_show_and_preview_sms",
            ],
        )
        self.add_to_fieldset(
            _("Postal"),
            [
                "sender_address_name",
                "sender_address",
                "reply_to_address_name",
                "reply_to_address",
                "acknowledgement_lre",
                "delivery_mode_lre",
                "priority_lre",
                "first_document",
                "buttons_show_and_preview_lre",
            ],
        )
        self.add_to_fieldset(
            _("Comment/Timestamps"),
            ["comment", "created_at", "updated_at"],
            classes=("wide", "collapse"),
        )
        self.add_to_fieldset(
            _("Configs"),
            ["metadata", "additional_context", "additional_config"],
            classes=("wide", "collapse"),
        )

    def _preview_buttons(self, obj, preview_type):
        """Show and Preview buttons for a given type (email, sms, postal)."""
        preview_url = reverse("django_pymissive:preview_form", args=["campaign"])
        buttons_html = []
        if obj.pk:
            base_url = reverse("django_pymissive:preview", args=["campaign", obj.pk])
            buttons_html.append(
                format_html(
                    '<a class="button" href="{}?type={}" target="_blank">{}</a>',
                    base_url,
                    preview_type,
                    _("Show"),
                )
            )
        pk_param = f"&pk={obj.pk}" if obj.pk else ""
        buttons_html.append(
            format_html(
                '<button type="submit" form="missivecampaign_form" formaction="{}?type={}{}" formmethod="post" formtarget="_blank" class="button" name="_preview" value="{}" style="margin-left: 10px;">{}</button>',
                preview_url,
                preview_type,
                pk_param,
                preview_type,
                _("Preview"),
            )
        )
        return mark_safe(" ".join(str(btn) for btn in buttons_html))  # nosec B703 B308

    def buttons_show_and_preview_email(self, obj):
        return self._preview_buttons(obj, "email")

    buttons_show_and_preview_email.short_description = _("Email: Show and Preview")

    def buttons_show_and_preview_sms(self, obj):
        return self._preview_buttons(obj, "sms")

    buttons_show_and_preview_sms.short_description = _("SMS: Show and Preview")

    def buttons_show_and_preview_lre(self, obj):
        return self._preview_buttons(obj, "postal")

    buttons_show_and_preview_lre.short_description = _("LRE: Show and Preview")

    def subject_display(self, obj):
        missive_recipients = [
            format_lazy(_("{} missive(s)"), obj.count_missive),
            format_lazy(_("{} recipient(s)"), obj.count_recipient),
        ]
        return self.format_with_help_text(obj.subject, " | ".join(str(s) for s in missive_recipients))

    def types_display(self, obj):
        """Display missive count per type."""
        labels = []
        related_attachment = [
            format_lazy(_("{} related(s)"), obj.count_related_object),
            format_lazy(_("{} attachment(s)"), obj.count_attachment),
        ]
        for type_key, type_label in MISSIVE_TYPES.items():
            count = getattr(obj, f"count_type_{type_key}", 0)
            if count:
                labels.append(
                    self.format_label(f"{count} {type_label}", size="small", label_type="primary")
                )
        tpl = mark_safe(" ".join(str(label) for label in labels)) if labels else "-"
        return self.format_with_help_text(tpl, " | ".join(str(s) for s in related_attachment))


    types_display.short_description = _("Types")

    def stats_display(self, obj):
        """Display missive/recipient counts and status percentages."""
        related_counts = [
            format_lazy(_("{} missive(s)"), obj.count_missive),
            format_lazy(_("{} event(s)"), obj.count_event),
        ]
        rates = [
            self.format_label(
                format_lazy(_("{}% failed"), f"{getattr(obj, 'pct_recipient_failed', 0):.0f}"),
                size="small",
                label_type="danger",
            ),
            self.format_label(
                format_lazy(_("{}% success"), f"{getattr(obj, 'pct_recipient_success', 0):.0f}"),
                size="small",
                label_type="success",
            ),
            self.format_label(
                format_lazy(_("{}% processing"), f"{getattr(obj, 'pct_recipient_processing', 0):.0f}"),
                size="small",
                label_type="warning",
            ),
        ]
        return self.format_with_help_text(mark_safe(" ".join(rates)), " | ".join(str(s) for s in related_counts))

    stats_display.short_description = _("Stats")

    def last_send_date_display(self, obj):
        """Display last send date from annotated queryset."""
        return getattr(obj, "last_send_date", None) or "-"

    last_send_date_display.short_description = _("Last send date")

    def last_ended_at_display(self, obj):
        """Display last ended at from annotated queryset."""
        return getattr(obj, "last_ended_at", None) or "-"

    last_ended_at_display.short_description = _("Last ended at")

    def has_start_campaign_permission(self, request, obj=None):
        return obj and obj.pk

    @admin_boost_action("start_campaign", _("Start campaign"))
    def handle_start_campaign(self, request, object_id):
        object_id = unquote(object_id)
        obj = self.get_object(request, object_id)
        return redirect(reverse("admin:django_pymissive_missivecampaign_start_campaign", args=[obj.pk]))

    @admin_boost_view("confirm", _("Start campaign"), hidden=True)
    def start_campaign(self, request, obj, confirmed=False):
        if not confirmed:
            return {"confirm": _("Are you sure you want to start this campaign?")}
        obj.start_campaign()
        messages.success(request, _("Campaign started successfully."))
        return redirect(reverse("admin:django_pymissive_missivecampaign_changelist"), args=[obj.pk])

    @admin_boost_view("redirect", _("Show missives"))
    def handle_show_missives(self, request, obj):
        url = reverse("admin:django_pymissive_missive_changelist")
        url += f"?campaign={obj.pk}"
        return url

    @admin_boost_view("redirect", _("Send missive"))
    def handle_send_missive(self, request, obj):
        url = reverse("admin:django_pymissive_missive_add")
        url += f"?campaign={obj.pk}"
        return url


"""Manager for missive webhooks."""

from urllib.parse import urlparse

from virtualqueryset.managers import VirtualManager


class MissiveWebhookManager(VirtualManager):
    """Manager for missive webhooks."""

    def get_queryset(self, provider=None):
        if provider is None:
            data = []
        else:
            if isinstance(provider, str):
                from ..models.provider import MissiveProviderModel

                provider = MissiveProviderModel.objects.get(name=provider)
            provider._provider.call_service("retrieve_webhooks")
            data = provider._provider.get_service_normalize("retrieve_webhooks")
            for item in data:
                item["provider"] = provider
                # Derive scheme and domain from url for edit form
                if url := item.get("url"):
                    parsed = urlparse(url)
                    item.setdefault("scheme", parsed.scheme or "https")
                    item.setdefault("domain", parsed.netloc)
        return self.queryset_class(model=self.model, data=data)



"""
Translation catalog for pymissive.config strings.

Auto-generated by: ./manage.py update_translation_catalog
Do not edit manually. Run update_translation_catalog after config changes.
"""

from django.utils.translation import gettext_lazy as _

# Strings from pymissive.config - used by makemessages for extraction

_("A request to send the missive has been received.")
_("Accepted")
_("Accepted by the provider for processing.")
_("Acknowledge button")
_("Acknowledgement of receipt")
_("Address")
_("Address format is invalid or malformed.")
_("Application")
_("Archived")
_("Archived by the provider.")
_("Attempted delivery")
_("Authenticated acknowledgement")
_("Automated voice call")
_("Basic delivery")
_("Being prepared for sending.")
_("Blocked")
_("Branded App messaging (WhatsApp, Slack, etc.)")
_("Cancelled")
_("Carrier rejected")
_("Clicked")
_("Connection refused by the server at SMTP level.")
_("Could not be delivered for unspecified reasons.")
_("Currently being processed by the system.")
_("Deferred")
_("Delivered")
_("Delivery blocked by the provider due to policies or security rules.")
_("Delivery was attempted.")
_("Deposit proof")
_("Deposit proof received from carrier.")
_("Domain not found")
_("Draft")
_("Dropped")
_("Economic")
_("eDelivery / eRegistered mail")
_("eIDAS qualified signature")
_("Electronic registered email (ERE)")
_("Electronic registered letter (LRE)")
_("Email")
_("Error")
_("Error occurred while processing")
_("Express")
_("Failed")
_("General failure to deliver.")
_("Hard bounce")
_("Has not been processed yet.")
_("High")
_("Identified as spam by the recipient or provider filters.")
_("In-app notification")
_("Intentionally dropped by the system, often due to suppression rules.")
_("Invalid")
_("Low")
_("Mailbox full")
_("Marketing email")
_("Message sent / delivered. No proof of reading or identity.")
_("MFA / OTP")
_("Mobile push notification")
_("Normal")
_("Official confirmation of delivery for legal/transactional missives.")
_("Opened")
_("Pending")
_("Permanent failure: cannot be delivered (e.g., address does not exist).")
_("Phone")
_("Premium")
_("Prepare")
_("Processed")
_("Processed by the provider.")
_("Processing")
_("Proof of content")
_("Proof of content received.")
_("Proofs of Delivery")
_("Qualified / legally binding acknowledgement")
_("Queued")
_("Queued and waiting for delivery.")
_("RCS (Rich SMS)")
_("Read")
_("Receipt + high-level verified identity + Qualified signature.")
_("Receipt confirmed. Authenticated identity (login, OTP, SSO).")
_("Receipt confirmed. Electronic signature. Full traceability.")
_("Recipient clicked a link in the missive.")
_("Recipient confirms receipt. No strong identity verification.")
_("Recipient explicitly marked as spam.")
_("Recipient opened the missive (tracked via open pixel).")
_("Recipient read the missive (may include extended tracking).")
_("Recipient unsubscribed from further missives.")
_("Recipient's carrier/provider rejected due to reputation or policy.")
_("Recipient's domain does not exist or cannot be resolved.")
_("Recipient's mailbox is full, causing a temporary delivery failure.")
_("Refused")
_("Rejected")
_("Rejected by the recipient's server (generic rejection).")
_("Reply email")
_("Request")
_("Saved as draft, not yet sent.")
_("Scheduled")
_("Scheduled to be sent at a future time.")
_("Secure portal")
_("Sending was cancelled before delivery.")
_("Sent")
_("Signed acknowledgement")
_("Simple or advanced e-signature")
_("SMS")
_("Soft bounce")
_("Spam")
_("Spam report")
_("Strong legal value (eIDAS)")
_("successfully delivered to the recipient's mailbox.")
_("Successfully sent from the sender server.")
_("Suppressed")
_("Suppressed by the system due to prior bounces or preferences.")
_("Technical only")
_("Temporary delivery failure, will retry later.")
_("Temporary failure: mailbox full, server busy, or other transient issues.")
_("Undelivered")
_("Unknown")
_("Unknown event.")
_("Unsubscribe")
_("Untreated")
_("Urgent")
_("User account")
_("Waiting to be processed for sending.")



"""Signal handlers for django_pymissive."""

from django.db.models.signals import post_save
from django.dispatch import receiver

from .models.event import MissiveEvent


@receiver(post_save, sender=MissiveEvent)
def trigger_billings_on_event(sender, instance, created, **kwargs):
    """Call get_billings on the missive after an event is saved."""
    if instance.missive_id and instance.missive.can_billings():
        instance.missive.get_billings()



from django.conf import settings


def get_campaign_backend():
    backend = getattr(settings, "CAMPAIGN_TASK_BACKEND", "sync")

    if backend == "celery":
        from .celery import CeleryBackend

        return CeleryBackend()
    elif backend == "rq":
        from .django_rq import RQBackend

        return RQBackend()
    elif backend == "thread":
        from .thread import ThreadBackend

        return ThreadBackend()
    else:
        from .sync import SyncBackend

        return SyncBackend()



"""Admin for webhook model."""

from urllib.parse import unquote

from django.contrib import admin
from django.http import HttpResponseRedirect
from django.utils.translation import gettext_lazy as _

from django_boosted import AdminBoostModel

from ..models.provider import MissiveProviderModel
from ..models.webhook import MissiveWebhook


class ProviderListFilter(admin.SimpleListFilter):
    """Custom filter for provider field."""

    title = _("Provider")
    parameter_name = "provider"

    def lookups(self, request, _model_admin):
        """Return list of providers as filter options."""
        try:
            # Get the provider field from the model
            provider_field = MissiveWebhook._meta.get_field("provider")
            # Use the field's method to get choices
            choices = provider_field.get_provider_choices()
            # Remove the empty choice
            return [choice for choice in choices if choice[0]]
        except Exception:
            return []

    def queryset(self, request, queryset):
        """Filter queryset by provider."""
        return queryset


@admin.register(MissiveWebhook)
class MissiveWebhookAdmin(AdminBoostModel):
    """Admin for missive webhooks."""

    list_display = [
        "id",
        "provider",
        "type",
        "url",
        "created_at",
        "updated_at",
    ]
    list_filter = [ProviderListFilter, "type"]
    search_fields = [
        "url",
        "description",
    ]
    readonly_fields = ["id", "webhook_id", "url", "created_at", "updated_at"]

    def log_addition(self, request, _obj, _message):
        pass

    def log_change(self, request, _obj, _message):
        pass

    def log_deletion(self, request, _obj, _object_repr):
        pass

    def has_add_permission(self, request):
        return True

    def has_change_permission(self, request, obj=None):
        if obj:
            provider = obj.get_provider()
            return hasattr(provider._provider, f"update_webhook_{obj.type}")
        return False

    def has_delete_permission(self, request, obj=None):
        if obj:
            provider = obj.get_provider()
            return hasattr(provider._provider, f"delete_webhook_{obj.type}")
        return False

    def get_readonly_fields(self, request, obj=None):
        if obj:
            readonly = list(self.readonly_fields)
            return readonly + ["provider"]
        return self.readonly_fields

    def change_fieldsets(self):
        self.add_to_fieldset(
            None,
            [
                "provider",
                "type",
                "scheme",
                "domain",
            ],
        )
        self.add_to_fieldset(_("Infos"), ["webhook_id", "url", "created_at", "updated_at"])

    def changelist_view(self, request, extra_context=None):
        if "provider" not in request.GET:
            providers = MissiveProviderModel.objects.all()
            if providers:
                params = request.GET.copy()
                params["provider"] = providers[0].name
                return HttpResponseRedirect(f"{request.path}?{params.urlencode()}")
        return super().changelist_view(request, extra_context=extra_context)

    def get_object(self, request, object_id, _from_field=None):
        webhook_id = unquote(object_id)
        provider = webhook_id.split("-")[0]
        qs = self.model.objects.get_queryset(provider)
        return next(
            (item for item in qs if str(item.webhook_id) == str(webhook_id)), None
        )

    def get_queryset(self, request):
        if provider := request.GET.get("provider"):
            return self.model.objects.get_queryset(provider)
        return self.model.objects.none()



"""Config model for default provider per missive type."""

from django.db import models
from django.utils.translation import gettext_lazy as _

from .choices import MissiveType
from django_providerkit import ProviderField

class MissiveConfig(models.Model):
    """Default provider configuration per missive type.
    When a missive has no provider set, the default for its type is used.
    Allows live switching (e.g. Brevo down -> change to SendGrid).
    """

    missive_type = models.CharField(
        max_length=50,
        choices=MissiveType.choices,
        unique=True,
        verbose_name=_("Missive type"),
        help_text=_("Type of missive (email, sms, postal, etc.)"),
    )
    default_provider = ProviderField(
        package_name="pymissive",
        blank=True,
        verbose_name=_("Default provider"),
        help_text=_("Provider name (e.g. brevo, sendgrid). Leave empty for no default."),
    )

    class Meta:
        verbose_name = _("Configuration")
        verbose_name_plural = _("Configurations")
        ordering = ["missive_type"]

    def __str__(self):
        return f"{self.get_missive_type_display()} → {self.default_provider or '-'}"



from django.http import FileResponse, Http404, HttpResponse
from django.shortcuts import redirect
from django.views.generic import DetailView

from ..models.attachment import MissiveBaseAttachment

_ATTACHMENT_QUERYSETS = {
    "missive": MissiveBaseAttachment.objects.filter(missive__isnull=False, campaign__isnull=True),
    "campaign": MissiveBaseAttachment.objects.filter(campaign__isnull=False, missive__isnull=True),
}


class MissiveAttachmentDownloadView(DetailView):
    """Download an attachment by id, scoped to missive or campaign."""

    model = MissiveBaseAttachment

    def get_queryset(self):
        key = self.kwargs.get("campaign_or_missive", "")
        qs = _ATTACHMENT_QUERYSETS.get(key)
        if qs is None:
            raise Http404
        return qs

    def _build_response(self, attachment_obj, attachment):
        """Build HTTP response from attachment object and attachment content."""
        if hasattr(attachment, "read") and hasattr(attachment, "name"):
            attachment.open("rb")
            name = (attachment.name and attachment.name.split("/")[-1]) or "unnamed_attachment"
            return FileResponse(attachment, as_attachment=True, filename=name)
        if isinstance(attachment, dict) and "url" in attachment:
            return redirect(attachment["url"])
        if isinstance(attachment, dict) and "content" in attachment:
            content = attachment["content"]
            name = attachment.get("name", "unnamed_attachment")
        else:
            content = attachment
            name = (getattr(attachment_obj, "metadata", None) or {}).get(
                "name", "unnamed_attachment"
            )
        response = HttpResponse(content, content_type="application/octet-stream")
        response["Content-Disposition"] = f'attachment; filename="{name}"'
        return response

    def get(self, request, *args, **kwargs):
        attachment_obj = self.get_object()
        attachment = attachment_obj.get_attachment()
        return self._build_response(attachment_obj, attachment)



from django.db import models
from django.utils.translation import gettext_lazy as _

from phonenumber_field.modelfields import PhoneNumberField
from django_geoaddress.fields import GeoaddressField

from .base import CommentTimestampedModel
from .choices import (
    MissiveRecipientType,
    MissiveStatus,
    status_from_event_counts,
    MissiveSupport,
)
from ..managers.recipient import (
    MissiveRecipientManager,
    MissiveRecipientEmailManager,
    MissiveRecipientPhoneManager,
    MissiveRecipientAddressManager,
    MissiveRecipientApplicationManager,
)


class MissiveRecipient(CommentTimestampedModel):
    """Recipient model"""

    missive = models.ForeignKey(
        "django_pymissive.Missive",
        on_delete=models.CASCADE,
        related_name="to_missiverecipient",
        verbose_name=_("Missive"),
        help_text=_("Missive"),
    )
    recipient_support = models.CharField(
        max_length=255,
        choices=MissiveSupport.choices,
        blank=True,
        null=True,
        verbose_name=_("Recipient Model"),
        help_text=_("Model of recipient"),
    )
    recipient_type = models.CharField(
        max_length=20,
        choices=MissiveRecipientType.choices,
        default=MissiveRecipientType.RECIPIENT,
        verbose_name=_("Recipient Type"),
        help_text=_("Type of recipient"),
    )
    status = models.CharField(
        max_length=20,
        choices=MissiveStatus.choices,
        default=MissiveStatus.DRAFT,
        verbose_name=_("Status"),
        help_text=_("Current status of the missive"),
    )
    name = models.CharField(
        max_length=255,
        blank=True,
        verbose_name=_("Name"),
        help_text=_("Full name or company name"),
    )

    email = models.EmailField(
        blank=True,
        null=True,
        verbose_name=_("Recipient Email"),
        help_text=_("Recipient's email address"),
    )
    phone = PhoneNumberField(
        max_length=20,
        blank=True,
        null=True,
        verbose_name=_("Phone"),
        help_text=_("Phone number"),
    )
    address = GeoaddressField(
        blank=True,
        null=True,
        verbose_name=_("Address"),
        help_text=_("Address"),
    )
    notification_id = models.CharField(
        max_length=255,
        blank=True,
        null=True,
        verbose_name=_("Notification ID"),
        help_text=_("Notification ID"),
    )

    external_id = models.CharField(
        max_length=255,
        blank=True,
        null=True,
        verbose_name=_("External ID"),
        help_text=_("External identifier from the provider"),
    )

    sent_at = models.DateTimeField(
        null=True,
        blank=True,
        verbose_name=_("Sent At"),
        help_text=_("When the missive was sent"),
    )
    delivered_at = models.DateTimeField(
        null=True,
        blank=True,
        verbose_name=_("Delivered At"),
        help_text=_("When the missive was delivered"),
    )
    objects = MissiveRecipientManager()

    class Meta:
        verbose_name = _("Recipient")
        verbose_name_plural = _("Recipients")
        ordering = ["name"]

    def __str__(self):
        return f"{self.name} ({self.target})"

    @property
    def target(self):
        return self.email or self.phone or self.address

    def get_serialized_data(self):
        return {
            "id": str(self.id),
            "name": self.name,
            "email": self.email,
            "phone": self.phone,
            "address": self.address,
            "notification_id": self.notification_id,
            "external_id": self.external_id,
        }

    @property
    def can_be_modified(self):
        return self.missive.can_be_modified

    def set_status(self):
        from ..models.event import MissiveEvent

        success_count, processing_count, failed_count = MissiveEvent.objects.get_event_counts(recipient=self)
        status = status_from_event_counts(success_count, processing_count, failed_count)
        if status != self.status:
            self.status = status
            self.save(update_fields=["status"])

class MissiveRecipientEmail(MissiveRecipient):
    objects = MissiveRecipientEmailManager()

    class Meta:
        proxy = True
        verbose_name = _("Email Recipient")
        verbose_name_plural = _("Email Recipients")

    def save(self, *args, **kwargs):
        if not self.recipient_support:
            self.recipient_support = MissiveSupport.EMAIL
        super().save(*args, **kwargs)


class MissiveRecipientPhone(MissiveRecipient):
    objects = MissiveRecipientPhoneManager()

    class Meta:
        proxy = True
        verbose_name = _("Phone Recipient")
        verbose_name_plural = _("Phone Recipients")

    def save(self, *args, **kwargs):
        if not self.recipient_support:
            self.recipient_support = MissiveSupport.PHONE
        super().save(*args, **kwargs)


class MissiveRecipientAddress(MissiveRecipient):
    objects = MissiveRecipientAddressManager()

    class Meta:
        proxy = True
        verbose_name = _("Address Recipient")
        verbose_name_plural = _("Address Recipients")

    def save(self, *args, **kwargs):
        if not self.recipient_support:
            self.recipient_support = MissiveSupport.ADDRESS
        super().save(*args, **kwargs)


class MissiveRecipientApplication(MissiveRecipient):
    objects = MissiveRecipientApplicationManager()

    class Meta:
        proxy = True
        verbose_name = _("Application Recipient")
        verbose_name_plural = _("Application Recipients")

    def save(self, *args, **kwargs):
        if not self.recipient_support:
            self.recipient_support = MissiveSupport.APPLICATION
        super().save(*args, **kwargs)


"""Views for Django Missive."""

from .attachment import (
    MissiveAttachmentDownloadView,
)
from .preview import (
    PreviewFormView,
    PreviewView,
)
from .webhook import WebhookView

__all__ = [
    "MissiveAttachmentDownloadView",
    "PreviewFormView",
    "PreviewView",
    "WebhookView",
]



from django.utils import timezone

from ..models.campaign import MissiveScheduledCampaign


def run_campaign(campaign_id):
    scheduled = MissiveScheduledCampaign.objects.get(
        id=campaign_id, send_date__isnull=True
    )
    scheduled.send_date = timezone.now()
    scheduled.save()
    try:
        scheduled.run_campaign()
    finally:
        # Clear processing flag so campaign can be restarted if needed
        campaign = scheduled.campaign
        metadata = dict(campaign.metadata or {})
        metadata.pop("processing", None)
        campaign.metadata = metadata
        campaign.save(update_fields=["metadata"])
    scheduled.ended_at = timezone.now()
    scheduled.save()
